define(function (require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');
    var URL = require('readnovelm/js/common/utils/url.js');

    var tplExtends = require('readnovelm/js/common/utils/tplExtends.js');
    var merge = require('readnovelm/js/common/page/merge.js');
    // 侧边栏切换

    var elBookDetailWrapper = $('#bookDetailWrapper');
    exports = module.exports = merge({
        //mixins: [lazyLoadMixin, urlMixin, limitFreeTimeMixin],
        params: { // ajax查询参数对象
            bookId: URL.getParamByName('bookId')
        },

        init: function () {
            var self = this;
            this.fillData(false, true);
            this.refreshEvent();
            this.bindCommentEvent();
            this.visibilityEvent();
        },

        visibilityEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', {name: 'browserVisibilityChange'});
            qdsdk.addEventListener('browserVisibilityChange', function (result) {
                if(result.data.visibility) {
                    self.fillData(false, false, true);
                }
            });
        },

        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', {name: 'refreshEvent'});
            qdsdk.addEventListener('refreshEvent', function (result) {
                if(result.result == 0) {
                    // true表示是通过refresh触发的
                    self.fillData(true);
                }
            });
        },


        bindCommentEvent: function () {
            var self = this;
            $('.book-li-write-comment').on('click', function (e) {
                e.preventDefault();
                qdsdk.invokeClient('app', 'checkLogin', {}, function (result) {
                    if(result.data.loginState) {
                        qdsdk.invokeClient('app', 'open', {url: baseLinkUrl + 'views/page/book/comment.html?pkgid=4&bookId=' + self.params.bookId });
                    } else {
                        qdsdk.app.openLogin();
                    }
                });
            });
        },

        fillData: function(refreshEvent, first, visibilityEvent) {
            var self = this;
            if (visibilityEvent) {
                qdsdk.invokeClient('app', 'isInBookshelf', {bookId: self.params.bookId}, function (result) {
                    if(result.data.isInBookshelf) {
                        $('#btnAddToBookshelf').text('已在书架').data('bookshelf-status', 1);
                    } else {
                        $('#btnAddToBookshelf').text('加入书架').data('bookshelf-status', 0);
                    }
                });
            }
            if (first) {
                self.fillTicketData();
                $.getJSON(ajaxUrl + 'book/getBookInfo', self.params, function (result) {
                    if (result.code === 0) {
                        var bookInfo = result.data.bookInfo;
                        if(refreshEvent) {
                            qdsdk.invokeClient('ui', 'cancelRefresh');
                            qdsdk.ui.toast('更新成功');
                        }
                        $('title').html(bookInfo.bookName);
                        $('.placeholder-box').remove();
                        var str = ejs.render('#tpl-book-detail.html', bookInfo);
                        $('#bookDetailBox').html(str);
                        //if(!refreshEvent) {
                            qdsdk.invokeClient('app', 'isInBookshelf', {bookId: bookInfo.bookId}, function (result) {
                                if(result.data.isInBookshelf) {
                                    $('#btnAddToBookshelf').text('已在书架').data('bookshelf-status', 1);
                                } else {
                                    $('#btnAddToBookshelf').text('加入书架').data('bookshelf-status', 0);
                                }
                            });
                        //}

                        // 初始化处理书描述
                        self.initSummary();
                        // 添加到书架功能
                        self.initAddToBookshelf(bookInfo);


                        self.loadBookComments();

                        // 作者还写过 todo
                        var otherBook = result.data.otherBook;
                        otherBook.eventid = 'hx_E07';
                        if(otherBook.length) {
                            var otherBookTemplate = ejs.render('#tpl-slide-li.html', { list: otherBook });
                            $('#otherOl').html(otherBookTemplate);
                            $('#otherBookModule').removeClass('hide');
                        }

                        // 同类推荐
                        var recommend = result.data.recommend;
                        recommend.eventid = 'hx_E08';
                        if(recommend.length) {
                            var recommendTemplate = ejs.render('#tpl-slide-li.html', { list: recommend });
                            $('#recommendOl').html(recommendTemplate);
                            $('#recommendModule').removeClass('hide');
                        }

                        $('.book-copyright p').html(result.data.copyright);
                        // 将浏览记录落到客户端
                        qdsdk.invokeClient('app', 'addBrowserHistoryBook', {bookId: bookInfo.bookId, bookName: bookInfo.bookName, author: bookInfo.authorName, description: bookInfo.description.substr(0, 200)});
                        qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                    }
                });
            } else {
                self.fillTicketData();
                self.loadBookComments();
                if (refreshEvent) {
                    qdsdk.invokeClient('ui', 'cancelRefresh');
                    qdsdk.ui.toast('更新成功');
                }
            }
        },

        // 处理顶部书描述展开收起
        initSummary: function () {
            // 展开时候为格式化排版
            var elSummary = $('#bookSummary');
            // 摘要内容元素
            var elSummaryContent = elSummary.find('content');
            // 存储格式化内容
            var htmlSummaryFormat = elSummary.find('textarea').val();
            // 存储纯text内容
            var textSummary = elSummaryContent.html();
            // 根据高度显示是否展示更多
            //计算行数
            var line = Math.ceil(parseFloat(elSummaryContent.height()) / parseFloat(elSummaryContent.css('line-height')));
            if (line > 4) {
            //if (elSummary[0] && elSummary[0].scrollHeight > elSummary[0].clientHeight) {
                elSummary.addClass('enabled');
                elSummary.on('click', function () {
                    var attrOpen = $(this).attr('open');
                    if (typeof attrOpen === 'string') {
                        $(this).removeAttr('open');
                        // 收起，显示纯文本
                        //elSummaryContent.html(textSummary);
                    } else {
                        $(this).attr('open', '');
                        // 展开，显示富文本
                        //elSummaryContent.html(htmlSummaryFormat);
                    }
                });
            }
        },

        initAddToBookshelf: function (bookInfo) {
            var self = this;
            var btnAddToBookshelf = $('#btnAddToBookshelf');

            btnAddToBookshelf.on('click', function () {
                if (btnAddToBookshelf.data('bookshelf-status') == 1) {
                    return false;
                }
                qdsdk.app.addToBookshelf({ bookId: bookInfo.bookId, bookName: bookInfo.bookName, author: bookInfo.authorName, lastChapterName: bookInfo.lastChapterName, bookStatus: bookInfo.bookStatus }, function (result) {
                    if(result.result == 0) {
                        if(result.data.addToBookshelf) {
                            qdsdk.ui.toast('加入书架成功');
                            btnAddToBookshelf.data('bookshelf-status', 1).html('已在书架');
                        }
                    } else {
                        qdsdk.ui.toast('加入书架失败，请稍候再试');
                    }
                });
            });
        },

        // 加载书评区
        loadBookComments: function () {
            var self = this;
            $.getJSON(ajaxUrl + 'forum/getCommentList', {bookId: self.params.bookId, pageNum: 1, pageSize: 3}, function (result) {
                // 书评
                var forumList = result.data;
                var _bookCoomentCnt = result.total;
                if (_bookCoomentCnt) {
                    $('.book-comment-count').html(_bookCoomentCnt);
                    var str = ejs.render('#tpl-review-list.html', { list: forumList,eventid: "hx_E14" });//TODO 待确认
                    $('#book-comment-container .module-title-desc').show();
                    $('#book-comment-list-container').html(str);
                    $('.book-li-more-comments').attr('href', 'javascript:qdsdk.invokeClient("app", "open", {url: "' + baseLinkUrl + 'views/page/book/reviews.html?pkgid=4&bookId=' + self.params.bookId + '"})');
                    $('.book-li-write-comment').attr('href', 'javascript:qdsdk.invokeClient("app", "open", {url: "' + baseLinkUrl + 'views/page/book/comment.html?pkgid=4&bookId=' + self.params.bookId + '"})');
                    $('#book-comment-list-container .null-x').eq(0).hide();
                    $('.book-li-write-comment').hide();
                    $('.book-li-more-comments').show();
                    $('.module-title-post-review.book-li-write-comment').show();
                } else {
                    $('.book-li-write-comment').show();
                    $('.book-li-more-comments').hide();
                    $('.module-title-post-review.book-li-write-comment').hide();
                    $('#book-comment-list-container .null-x').eq(0).hide();
                }

            })
        },
        fillTicketData: function () {
            var self = this;
            $.getJSON(ajaxUrl + 'ticket/info', {bookId: self.params.bookId}, function (result) {
                result.data.yeventid = "hx_E11";//TODO 待确认
                result.data.teventid = "hx_E12";//TODO 待确认
                result.data.seventid = "hx_E13";//TODO 待确认
                var str = ejs.render('#tpl-ticket.html', {
                    data: result.data,
                    bookId: "" + self.params.bookId
                });
                $('#payTicketsX').html(str);
            });
        }
    });
});
