define(function (require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');

    // URL处理工具
    var URL = require('../../../static/js/common/utils/url.js');

    // 页面继承方法
    var merge = require('../../../static/js/common/page/merge.js');

    var lazyLoadMixin = require('../../../static/js/common/mixin/lazyLoad.js');

    // loading交互
    require('../../../static/js/common/components/Loading.js');

    var data = {
        book: g_data.book,
    };

    exports = module.exports = merge({
        mixins: [lazyLoadMixin],

        param: {
            bookId: URL.getParamByName('bookId')
        },

        init: function () {
            this.fillData();
            this.refreshEvent();
            this.visibilityEvent();
        },

        visibilityEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', { name: 'browserVisibilityChange' });
            qdsdk.addEventListener('browserVisibilityChange', function (result) {
                if (result.data.visibility) {
                    self.fillData();
                }
            });
        },

        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', { name: 'refreshEvent' });
            qdsdk.addEventListener('refreshEvent', function (result) {
                if (result.result == 0) {
                    // true表示是通过refresh触发的
                    self.fillData(true);
                }
            });
        },

        // 填充数据
        fillData: function (refreshEvent) {
            var self = this;
            self.fillTicketData();
            $.getJSON(ajaxUrl + 'book/getLastPageInfo', {
                bookId: self.param.bookId,
            }, function (result) {
                if (result.code === 0) {
                    // result.data.bookInfo = g_data.book;
                    // 书评区
                    if (refreshEvent) {
                        qdsdk.invokeClient('ui', 'cancelRefresh');
                        qdsdk.ui.toast('更新成功');
                    }
                    $('title').html(result.data.bookName);

                    if (result.data.isFinish) {
                        $('#bookStatus').text('全书完');
                    } else {
                        $('#bookStatus').text('未完待续');
                    }

                    var forumList = result.data.forumList;

                    var forumListTemplate = ejs.render('#tpl-review-list.html', { list: forumList.list,eventid: "hx_F34" });
                    if (forumList.total) {
                        $('#loadX1 .null-x').eq(0).hide();
                        $('#loadX1').html(forumListTemplate);
                    } else {
                        $('#loadX1 .null-x').eq(0).hide();
                        $('#loadX1 .null-x2').show();
                    }

                    $('#totalCnt').html(forumList.total);
                    $('.book-li-more-comments').attr('href', 'javascript:qdsdk.invokeClient("app", "open", {url: "' + baseLinkUrl + 'views/page/book/reviews.html?pkgid=4&bookId=' + self.param.bookId + '"})');
                    $('.book-li-write-comment').attr('href', 'javascript:qdsdk.invokeClient("app", "open", {url: "' + baseLinkUrl + 'views/page/book/comment.html?pkgid=4&bookId=' + self.param.bookId + '"})');
                    if (!forumList.total) {
                        $('.book-li-more-comments').hide();
                        $('.book-li-write-comment').show();
                        $('.module-title-post-review.book-li-write-comment').hide();
                    }else{
                        $('.module-title-post-review.book-li-write-comment').show();
                    }

                    // 作者还写过 todo
                    var otherBook = result.data.otherBook;
                    otherBook.eventid = 'hx_F29';
                    if (otherBook.length) {
                        var otherBookTemplate = ejs.render('#tpl-slide-li.html', { list: otherBook });
                        $('#otherOl').html(otherBookTemplate);
                        $('#otherBookModule').removeClass('hide');
                    }

                    // 看过本书的人还看过其他的
                    var recommend = result.data.recommend;
                    recommend.eventid = 'hx_F30';
                    // 遗留问题，ejs2在模板里无法用this来访问传入的变量，所以此处依然用了ejs1来render
                    // var recommendTemplate = ejs.render('#tpl-slide-li.html', recommend);
                    if (recommend.length) {
                        var recommendTemplate = ejs.render('#tpl-slide-li.html', { list: recommend });
                        $('#recommendOl').html(recommendTemplate);
                        $('#recommendModule').removeClass('hide');
                    }
                    qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                }
            });
        },
        fillTicketData: function () {
            var self = this;
            $.getJSON(ajaxUrl + 'ticket/info', { bookId: self.param.bookId }, function (result) {
                result.data.yeventid = "hx_F31";
                result.data.teventid = "hx_F32";
                result.data.seventid = "hx_F33";
                var str = ejs.render('#tpl-ticket.html', {
                    data: result.data,
                    bookId: '' + self.param.bookId
                });
                $('#payTicketsX').html(str);
            });
        }
    });
});
