define(function (require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');

    // URL处理工具
    var URL = require('../../../static/js/common/utils/url.js');
    var Popup = require('../../../static/js/common/components/Popup.js');

    // 页面继承方法
    var merge = require('../../../static/js/common/page/merge.js');

    var infiniteScrollMixin = require('../../../static/js/common/mixin/infiniteScroll.js');

    exports = module.exports = merge({
        mixins: [infiniteScrollMixin],

        params: { // ajax查询参数对象
            bookId: URL.getParamByName('bookId'),
            commentId: URL.getParamByName('commentId'),
            pageNum: 1,
        },

        init: function () {
            this.fillData();
            this.initReply();
            this.refreshEvent();
            // 举报事件
        },

        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', { name: 'refreshEvent' });
            qdsdk.addEventListener('refreshEvent', function (result) {
                if (result.result == 0) {
                    // true表示是通过refresh触发的
                    self.fillData(true);
                }
            });
        },

        reportEvent: function () {
            var self = this;
            $('.btn-report').on('click', function (e) {
                e.preventDefault();
                var commentId = self.params.commentId;
                var commentType = '' + $(this).data('comment-type');
                qdsdk.invokeClient('app', 'openReport', { commentId: commentId, commentType: commentType });
            });
        },

        bindCommentEvent: function () {
            var self = this;
            $('.comm-fixed-btn').on('click', function (e) {
                e.preventDefault();
                qdsdk.invokeClient('app', 'checkLogin', {}, function (result) {
                    if (result.data.loginState) {
                        qdsdk.invokeClient('app', 'open', { url: baseLinkUrl + 'views/page/book/comment.html?pkgid=4&bookId=' + self.params.bookId });
                    } else {
                        qdsdk.app.openLogin();
                    }
                });
            });
        },

        onInfiniteScroll: function () {
            this.params.pageNum++;
            this.fillData();
        },

        initRaty: function () {
            var bookId = this.params.bookId;

            var ACTIVE = 'red';

            function isLiked(el) {
                return el.hasClass(ACTIVE);
            }

            function toggleStatus(el) {
                var elOutput = el.find('output'); // 数值元素
                var valOutput = elOutput.html(); // 值

                if (isLiked(el)) {
                    // 取消赞
                    el.removeClass(ACTIVE).attr('title', '赞一个');
                    // 数值变化
                    elOutput.html(+valOutput - 1);
                } else {
                    el.addClass(ACTIVE).attr('title', '取消赞');
                    // 数值变化
                    elOutput.html(+valOutput + 1);
                }
            }

            // 点赞
            $('#reviews').on('click', '.jsLike', function () {
                var $this = $(this);

                if ($this.css('opacity') < 1) {
                    return;
                }
                $this.css('opacity', 0.5);

                var params = {
                    bookId: bookId,
                    rateId: $this.data('rate-id'),
                    status: isLiked($this) ? 0 : 1,
                };

                $.post('/ajax/book/commentLike', params, function (result) {
                    $this.css('opacity', '');

                    if (result.code == 0) {
                        toggleStatus($this); // 失败的时候，取消刚才的操作
                    }
                });
            });
        },

        initReply: function () {
            var self = this;
            var $form = $('#commReplyPopup');
            var $content = $form.find('textarea');
            var $cancel = $('#commReplyCancel');

            var popup;

            $('#commReplyBtn').on('click', function () {
                if (popup) {
                    popup.show();
                } else {
                    popup = new Popup($form.removeAttr('hidden'), {
                        onShow: function () {
                            if ($content.val() == '') {
                                // 没有内容，动画结束自动focus
                                setTimeout(function () {
                                    $content[0].focus();
                                }, 300);
                            }
                        }
                    });

                    // 小米手机横竖屏切换的处理
                    if (/android.*xiaomi.*miuibrowser/i.test(navigator.userAgent)) {
                        window.onorientationchange = function () {
                            if (popup.el.aside.hasClass('active')) {
                                popup.el.aside.find('textarea')[0].blur();
                            }
                        };
                    }
                }
            });

            $cancel.on('click', function () {
                if (popup) {
                    popup.hide();
                }
            });

            // 保存新建分组
            $form.on('submit', function (event) {
                event.preventDefault();
                var content = $content.val().trim();

                if (!content) {
                    qdsdk.ui.toast('请输入回复内容');
                    $content.focus();
                    return;
                }
                $.post(ajaxUrl + 'forum/reply', {
                    bookId: self.params.bookId,
                    content: content,
                    commentId: self.params.commentId
                }, function (result) {
                    var result = JSON.parse(result);
                    if (result.code == 0) {
                        qdsdk.ui.toast('回复成功');
                        self.fillData();

                        // var $count = $('#replyCount');
                        // $count.text(+$count.text() + 1);

                        // var $list = $('#postListContainer');

                        // result.data.time = '刚刚';
                        // result.data.postContent = content;

                        // var $null = $('#nullPost');
                        // if ($null.length) {
                        //     $null.remove();
                        // }

                        // var $end = $('#endPost');
                        // if ($end.length) {
                        //     $end.remove();
                        // }

                        // var str = ejs.render('#tpl-post-list.html', {
                        //     data: {
                        //         postList: [result.data],
                        //         isLast: true,
                        //         pageNum: $null.length ? 1 : 0,
                        //     },
                        // });

                        // $list.append(str);
                    } else {
                        qdsdk.ui.toast(result.msg);
                    }
                });

                $cancel.trigger('click');
                $form[0].reset();
            });

            // $("#post-content-input").on("touchstart",function(e){
            //     this.nowScrollTop = document.body.scrollTop;
            //     $(".aside").css("position","absolute");
            //     $("#post-content-input").focus();
            // })
            // $("#post-content-input").on("blur",function(){
            //     $(".aside").css("position","fixed");
            //     window.scrollTo(0, this.nowScrollTop || 0);
            // })
            // document.getElementById("post-content-input").addEventListener('touchstart', function(e) {
            //     $(".aside").css("position","static");
            //     $("#post-content-input").focus();
            //     e.preventDefault();
            // }, false);
            // document.getElementById("post-content-input").addEventListener('blur', function(e) {
            //     $(".aside").css("position","fixed");
            // }, false);
        },

        fillData: function (refreshEvent) {
            var self = this;

            this.disableInfiniteScroll();

            $.getJSON(ajaxUrl + 'forum/getCommentDetail', this.params, function (result) {
                

                if (result.code == 0) {
                    result.data.pageNum = self.params.pageNum; // 后台没有返回，自行补齐

                    if (refreshEvent) {
                        qdsdk.invokeClient('ui', 'cancelRefresh');
                        qdsdk.ui.toast('更新成功');
                    }
                    var str = ejs.render('#tpl-review-detail.html', result.data.comment);
                    $('.comm-detail-x').html(str);

                    var str = ejs.render('#tpl-reply-list.html', { list: result.data.replyList });
                    $('#replyCount').html(result.total);
                    if (self.params.pageNum == 1) {
                        $('#replayListContainer').html(str);
                    } else {
                        $('#replayListContainer').append(str);
                    }
                    if (result.total == 0) {
                        $('#nullPost').show();
                    } else {
                        $('#nullPost').hide();
                    }
                    self.reportEvent();

                    self.scrollerSelector = $(".page-comment-detail");
                    self.initInfiniteScroll();
                    self.enableInfiniteScroll();
                    self.toggleInfiniteScroll(result.isLast);
                    // if(!result.isLast){
                    //     self.onInfiniteScroll();
                    // }
                    qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                }
            });
        },
        initReplayScroll: function () {
            $(".module-content").on('scroll', function (event) {
                if((event.srcElement.scrollHeight - event.srcElement.scrollTop < 600) && !result.isLast){
                    //this.onInfiniteScroll();//需要添加loadBar
                }
            })
        }
    });
});
