define(function (require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');

    // URL处理工具
    var URL = require('../../../static/js/common/utils/url.js');

    // 页面继承方法
    var merge = require('../../../static/js/common/page/merge.js');

    var infiniteScrollMixin = require('../../../static/js/common/mixin/infiniteScroll.js');

    exports = module.exports = merge({
        mixins: [infiniteScrollMixin],

        params: { // ajax查询参数对象
            bookId: URL.getParamByName('bookId'),
            pageNum: 1,
            type: URL.getParamByName('type') || 0
        },

        init: function () {
            this.initRaty();
            this.fillData();
            this.bindCommentEvent();
            this.refreshEvent();
            this.switchType();
            this.visibilityEvent();
        },

        visibilityEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', { name: 'browserVisibilityChange' });
            qdsdk.addEventListener('browserVisibilityChange', function (result) {
                if (result.data.visibility) {
                    self.fillData();
                }
            });
        },

        switchType: function () {
            var self = this;
            $('#commentNav a').on('click', function (e) {
                var elTag = $(this);
                if (elTag.hasClass('active')) {
                    return;
                }
                var type = elTag.data('type');
                var ol = $('#reviews' + type);
                self.params.type = type;
                $('#commentNav a.active').removeClass('active');
                elTag.addClass('active');

                self.params.pageNum = 1;
                if (ol.data('cached') === true) {
                    elTag.siblings().removeClass('active');
                    elTag.addClass('active');
                    ol.siblings('ol').attr('hidden', '');
                    ol.removeAttr('hidden');
                    if (ol.find('li').length == 0) {
                        $('#reviewsWrapper .null-x').show();
                    } else {
                        $('#reviewsWrapper .null-x').hide();
                    }
                } else {
                    self.showInfiniteScroll();
                    self.params.type = type;
                    // elRankWrapper.data('order-type', orderBy);
                    self.fillData();
                }
            });
        },

        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', { name: 'refreshEvent' });
            qdsdk.addEventListener('refreshEvent', function (result) {
                if (result.result == 0) {
                    // true表示是通过refresh触发的
                    self.fillData(true);
                }
            });
        },

        bindCommentEvent: function () {
            var self = this;
            $('.comm-fixed-btn').on('click', function (e) {
                e.preventDefault();
                qdsdk.invokeClient('app', 'checkLogin', {}, function (result) {
                    if (result.data.loginState) {
                        qdsdk.invokeClient('app', 'open', { url: baseLinkUrl + 'views/page/book/comment.html?pkgid=4&bookId=' + self.params.bookId });
                    } else {
                        qdsdk.app.openLogin();
                    }
                });
            });
        },

        onInfiniteScroll: function () {
            this.params.pageNum++;
            this.fillData();
        },

        initRaty: function () {
            var bookId = this.params.bookId;

            var ACTIVE = 'red';

            function isLiked(el) {
                return el.hasClass(ACTIVE);
            }

            function toggleStatus(el) {
                var elOutput = el.find('output'); // 数值元素
                var valOutput = elOutput.html(); // 值

                if (isLiked(el)) {
                    // 取消赞
                    el.removeClass(ACTIVE).attr('title', '赞一个');
                    // 数值变化
                    elOutput.html(+valOutput - 1);
                } else {
                    el.addClass(ACTIVE).attr('title', '取消赞');
                    // 数值变化
                    elOutput.html(+valOutput + 1);
                }
            }

            // 点赞
            $('#reviews').on('click', '.jsLike', function () {
                var $this = $(this);

                if ($this.css('opacity') < 1) {
                    return;
                }
                $this.css('opacity', 0.5);

                var params = {
                    bookId: bookId,
                    rateId: $this.data('rate-id'),
                    status: isLiked($this) ? 0 : 1,
                };

                $.post('/ajax/book/commentLike', params, function (result) {
                    $this.css('opacity', '');

                    if (result.code === 0) {
                        toggleStatus($this); // 失败的时候，取消刚才的操作
                    }
                });
            });
        },

        fillData: function (refreshEvent) {
            var self = this;

            var elReviews = $('#reviews' + self.params.type);

            this.disableInfiniteScroll();
            self.fillTicketData();

            $.getJSON(ajaxUrl + 'forum/getCommentList', this.params, function (result) {
                self.enableInfiniteScroll();

                if (result.code === 0) {
                    result.data.pageNum = self.params.pageNum; // 后台没有返回，自行补齐

                    if (refreshEvent) {
                        qdsdk.invokeClient('ui', 'cancelRefresh');
                        qdsdk.ui.toast('更新成功');
                    }
                    var str = ejs.render('#tpl-review-list.html', { list: result.data,eventid:"hx_K06" });
                    $('.comm-fixed-btn').attr('href', 'javascript:qdsdk.invokeClient("app", "open", {url: "' + baseLinkUrl + 'views/page/book/comment.html?pkgid=4&bookId=' + self.params.bookId + '"})');

                    if (self.params.pageNum === 1) {
                        elReviews.html(str);
                    } else {
                        elReviews.append(str);
                    }
                    elReviews.data('cached', true); // 标记为缓存
                    elReviews.siblings('ol').attr('hidden', '');
                    elReviews.removeAttr('hidden');
                    if (result.data.length == 0) {
                        $('#reviewsWrapper .null-x').show();
                    } else {
                        $('#reviewsWrapper .null-x').hide();
                    }

                    self.toggleInfiniteScroll(result.isLast);
                    qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                }
            });
        },
        fillTicketData: function () {
            var self = this;
            $.getJSON(ajaxUrl + 'ticket/info', { bookId: self.params.bookId }, function (result) {
                result.data.yeventid = "hx_K01";
                result.data.teventid = "hx_K02";
                result.data.seventid = "hx_K03";
                var str = ejs.render('#tpl-ticket.html', {
                    data: result.data,
                    bookId: '' + self.params.bookId
                });
                $('#payTicketsX').html(str);
            });
        }
    });
});
