define(function(require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');
    var URL = require('../../../static/js/common/utils/url.js');
    // 侧边栏切换
    var Aside = require('../../../static/js/common/components/Aside.js');
    var LazyLoad = require('../../../static/js/common/components/LazyLoad.js');
    // 模板扩展方法
    var tplExtends = require('../../../static/js/common/utils/tplExtends.js');

    // 模板扩展方法
    var merge = require('../../../static/js/common/page/merge.js');

    var filtersMixin = require('../../../static/js/common/mixin/filters.js');
    var lazyLoadMixin = require('../../../static/js/common/mixin/lazyLoad.js');
    var infiniteScrollMixin = require('../../../static/js/common/mixin/infiniteScroll.js');


    var container = $('#content');
    var elRankWrapper = $('#categoryListContainer');

    exports = module.exports = merge({
        mixins: [filtersMixin, lazyLoadMixin, infiniteScrollMixin],
        params: { // ajax查询参数对象
            pageNum: 1,
            orderBy: URL.getParamByName('orderBy') || elRankWrapper.data('order-by'),
            categoryId: URL.getParamByName('categoryId') || -1,
            subCategoryId: URL.getParamByName('subCategoryId') || -1,
            state: URL.getParamByName('state') || -1,
            attr: URL.getParamByName('attr') || -1,
            cnt: URL.getParamByName('cnt') || -1,
            updT: URL.getParamByName('updT') || -1
        },

        //侧边栏过滤条件
        filter: {},
        init: function() {
            // alert(elRankWrapper.data('order-yype'))
            this.initAside();
            this.switchSortOrder();
            this.initParams();


            this.fillData(false, true);
            this.refreshEvent();
            this.setOrderActive();
            this.initAsideFilter();
        },

        setOrderActive: function () {
            var orderBy = URL.getParamByName('orderBy') || 13;
            if(orderBy) {
                $('.sort-order-switcher .btn-tag').each(function () {
                    if($(this).data('value') == orderBy) {
                        $(this).addClass('active').siblings().removeClass('active');
                    }
                });
            }
            $('#filters .sort-li').each(function () {
                var opt = $(this).data('key');
                var urlOpt = URL.getParamByName(opt);
                if(urlOpt) {
                    $(this).find('.btn-tag').each(function () {
                        if($(this).data('value') == urlOpt) {
                            $(this).addClass('active').siblings().removeClass('active');
                        }
                    });
                }
            });
        },

        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', {name: 'refreshEvent'});
            qdsdk.addEventListener('refreshEvent', function (result) {
                if(result.result == 0) {
                    // true表示是通过refresh触发的
                    self.fillData(true);
                }
            });
        },

        initParams: function() {
             this.params.categoryId = URL.getParamByName('categoryId') || -1;
             this.params.subCategoryId = URL.getParamByName('subCategoryId') || -1;
        },

        // 侧边栏展开和收起效果
        initAside: function() {
             this.ASIDE = new Aside($('#asideTrigger'));
        },

        //侧边栏过滤筛选
        initAsideFilter: function() {
            var self = this;
            $('#sortByFilter').on('click', function() {
                $.extend(self.params, self.filter);
            });
        },

        //初始化筛选框数据
        onFilterInit: function(filterKey, filterValue) {
            this.params[filterKey] = URL.getParamByName(filterKey) || -1;
            this.filter[filterKey] = URL.getParamByName(filterKey) || -1;
        },

        //更新筛选框数据
        onFilterChange: function(filterKey, filterValue) {
            var self = this;
            this.filter[filterKey] = filterValue;
            self.gotoSearch();
        },

        // 取消筛选框数据
        onFilterCancel: function (filterKey) {
            var self = this;
            this.filter[filterKey] = undefined; // 通过 undefined 标记被清除
            self.gotoSearch();
        },

        onFiltersReset: function (filterKeys) {
            var self = this;

            filterKeys.forEach(function (filterKey) {
                self.filter[filterKey] = undefined; // 通过 undefined 标记被清除
            });
            self.gotoSearch();            
            $('#filters .btn-tag.active').removeClass('active');
        },

        gotoSearch: function() {
            var self = this;

            var url;
            self.params.pageNum = 1;
            Object.keys(self.params).forEach(function(key) {
                url = URL.setParam(key, self.params[key], url);
                if (key in self.filter) {
                    self.params[key] = self.filter[key];
                }
            });
            self.fillData();
            // self.ASIDE.hide()
        },
        //切换排序顺序
        switchSortOrder: function() {
            var self = this;
            $('.sort-order-switcher .btn-tag').on('click', function(e) {
                var elTag = $(this);
                if(elTag.hasClass('active')) {
                    return;
                }
                var orderBy = elTag.data('value');
                var elBooks = $('#books-' + orderBy);
                self.params.orderBy = orderBy;
                $('.sort-order-switcher a.active').removeClass('active');
                elTag.addClass('active');

                self.params.pageNum = 1;
                if (elBooks.data('cached') === true) {
                    elTag.siblings().removeClass('active');
                    elTag.addClass('active');
                    elBooks.siblings('.jsBooks').attr('hidden', '');
                    elBooks.removeAttr('hidden');
                } else {
                    self.showInfiniteScroll();
                    self.params.orderBy = orderBy;
                    elRankWrapper.data('order-type', orderBy);
                    self.fillData();
                }
            })
        },
        //滚动时加载更多
        onInfiniteScroll: function() {
            this.params.pageNum++;
            this.fillData();
        },

        // 动态设置页面title
        setTitle: function(title) {
            var curTitle = $('title').html();
            var nowTitle = URL.getParamByName('title') || title;
            if(title !== curTitle) {
                $('title').html(nowTitle)
            }
        },

        //加载更多数据
        fillData: function(refreshEvent, first) {
            var self = this;
            this.disableInfiniteScroll();
            var elBooks = $('#books-' + self.params.orderBy); // 标记为缓存
            var showLoading = self.params.pageNum === 1;
            if (showLoading) {
                qdsdk.invokeClient('ui', 'startRefresh');
            }
            $.getJSON(ajaxUrl + 'book/category', this.params, function(result) {
                self.enableInfiniteScroll();

                // 移除loading
                if (showLoading) {
                    qdsdk.invokeClient('ui', 'cancelRefresh');
                }
                if (result.code === 0) {
                    result.staticConf = g_data; // eslint-disable-line camelcase
                    result.domains = g_data.domains;
                    if(refreshEvent) {
                        qdsdk.invokeClient('ui', 'cancelRefresh');
                        qdsdk.ui.toast('更新成功');
                    }
                    self.setTitle(result.title);
                    var str = ejs.render('#tpl-init-list.html', result);
                    if (self.params.pageNum === 1) {
                        elBooks.html(str);
                    } else {
                        elBooks.append(str);
                    }
                    elBooks.data('cached', true); // 标记为缓存
                    elBooks.siblings('.jsBooks').attr('hidden', '');
                    elBooks.removeAttr('hidden');

                    var _toggleInfinit = (result.isLast == 1) ? true : false;
                    if(result.isLast == 1) {
                        $('#loadBar').html('已无更多数据')
                    }

                    self.toggleInfiniteScroll(_toggleInfinit);
                    self.updateLazyLoad();
                    qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                }
            });
            if(self.params.subCategoryId == -1 && first) {
                $.getJSON(ajaxUrl + 'book/getSubCategoryById', {categoryId: self.params.categoryId}, function(result) {
                    if(result.code == 0) {
                        var filterCategory = ejs.render('#tpl-filter-category.html', { list: result.data });
                        $('#filter-category').html(filterCategory);
                        setTimeout(function () {
                            $('#filter-category .btn-tag').on('click', function (e) {
                                e.preventDefault();
                                var ele = $(this);
                                if(ele.hasClass('active')) {
                                    self.onFilterCancel('subCategoryId');    
                                    setTimeout(function () {
                                        $('.jsTag').removeClass('active');  
                                    }, 500);                                
                                } else {
                                    self.onFilterChange('subCategoryId', ele.data('value'));
                                    ele.addClass('active').siblings().removeClass('active');
                                }
                            });
                        }, 1000);
                    }
                });
            } else {
            }
        }
    });
});
