/**
 * @Tab.js
 * @author zhangxinxu
 * @version
 * @Created: 15-06-12
 * @description form LuLu
 */

(function (global, factory) {
    if (typeof define === 'function' && (define.amd || define.cmd)) {
        define(factory);
    } else {
        factory();
    }
}(this, function () {
    'use strict';

    //var $ = window.Zepto;

    /**
     * 滚动或者切换时候的图片资源的延迟加载
     * @使用示例
     *  new LazyLoad($('#content'), {
     *     callback: function() {}
     *  });

     * 或者包装器用法
     * $('#content').lazyLoad({
     *   callback: function() {}
     * });
    **/

    $.fn.lazyLoad = function (options) {
        var defaults = {
            selector: 'img[data-src]',
            direction: 'vertical',      // 还可以是'horizontal', 'both'
            callback: function () {}
        };
        var params = $.extend({}, defaults, options || {});

        return $(this).each(function () {
            // 一些元素
            var container = $(this), isWindow = (window === this);
            // 获取属性
            var attribute = params.selector.split(']')[0].split('[')[1] || '';
            // 图片资源
            var elImages = isWindow ? $(params.selector) : container.find(params.selector);

            // 检测方向
            var direction = params.direction;

            // if (elImages.length === 0) {
            //     return false;
            // }

            // 容器等位置和尺寸
            var offsetContainer = isWindow ? {
                left: 0,
                right: container.width(),
                top: 0,
                bottom: container.height(),
                width: container.width(),
                height: container.height(),
            } : container.offset();

            // 滚动触发
            container.on('scroll', function (event) {
                if (event.isTrusted === false || event.hasOwnProperty('_args')) {
                    // 如果是不信任的滚动事情，则我们重新拉一遍图片
                    elImages = isWindow ? $(params.selector) : container.find(params.selector);
                }

                elImages.each(function () {
                    var elImg = $(this);
                    // 是否是图片
                    var isImg = elImg.is('img');
                    // 图片等一些数据
                    var offsetImg = elImg.offset();

                    if (isWindow) {
                        offsetImg.top = offsetImg.top - container.scrollTop();
                        offsetImg.left = offsetImg.left - container.scrollLeft();
                    }

                    // 最终处理等地址
                    var src = '';

                    if (attribute) {
                        src = elImg.attr(attribute);
                        if (!src) {
                            return;
                        }
                    }

                    if (offsetImg.width && offsetImg.height) {
                        // 水平或垂直任意一个在容器中即可
                        var posTopImg = offsetImg.top - offsetContainer.top;
                        var posBottomImg = offsetImg.top + offsetImg.height;
                        // 水平方向
                        var posLeftImg = offsetImg.left - offsetContainer.left;
                        var posRightImg = offsetImg.right - offsetContainer.right;

                        // 垂直方向是否在区间
                        var isVertical = (posTopImg >= 0 && posTopImg < offsetContainer.height) ||
                                (posBottomImg > 0 && posBottomImg <= offsetContainer.height);
                        // 水平方向是否在区间
                        var isHorizontal = (posLeftImg >= 0 && posLeftImg < offsetContainer.width) ||
                                (posRightImg > 0 && posRightImg <= offsetContainer.width);

                        if ((direction === 'vertical' && isVertical) || (direction === 'horizontal' && isHorizontal) || (direction === 'both' && isVertical && isHorizontal)) {
                            if (isImg) {
                                elImg.attr('src', src).on('load', function () {
                                    $(this).removeAttr(attribute);
                                });
                            }
                            if ($.isFunction(params.callback)) {
                                params.callback.call(container, elImg);
                            }
                        }
                    }
                });
            });
            // 默认触发一次
            container.trigger('scroll');
        });
    };

    var LazyLoad = function (container, options) {
        this.container = container;
        container.lazyLoad(options);
    };

    LazyLoad.prototype.trigger = function () {
        var container = this.container;
        container.trigger('scroll');
    };

    return LazyLoad;
}));
