/**
 * @Loading.js
 * @author zhangxinxu
 * @version
 * Created: 16-09-10
 */

(function (global, factory) {
    if (typeof define === 'function' && (define.amd || define.cmd)) {
        define(factory);
    } else {
        global.Loading = factory();
    }
}(this, function () {
    var CL = 'loading-x', CL_ICON = 'loading-icon', CL_BUTTON_LOADING = 'loading', CL_ANIMATION = 'loading-animation';

    var body = $('body');

    var svg = '<svg class="loading-svg" width="24" height="24" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg"><g stroke-width="1.5" fill="none" stroke-linecap="round" stroke-linejoin="round"><path d="M1 2.998C1 1.894 1.887 1 3.006 1h3.492c.83 0 1.899.545 2.389 1.22l6.226 8.56c.49.674.53 1.78.084 2.482l-5.394 8.476C9.36 22.435 8.332 23 7.495 23H2.998A2 2 0 0 1 1 21.002V18.5c0-.828.42-2.025.938-2.671l2.437-3.047c.345-.431.35-1.125 0-1.562L1.938 8.172C1.42 7.525 1 6.335 1 5.502V2.997z"/><path d="M23 2.998A1.995 1.995 0 0 0 20.994 1h-3.492c-.83 0-1.899.545-2.389 1.22l-6.226 8.56c-.49.674-.53 1.78-.084 2.482l5.394 8.476c.443.697 1.471 1.262 2.308 1.262h4.497A2 2 0 0 0 23 21.002V18.5c0-.828-.42-2.025-.938-2.671l-2.437-3.047c-.345-.431-.35-1.125 0-1.562l2.437-3.047c.518-.647.938-1.837.938-2.67V2.997z"/></g></svg>';

	/**
     * 是否正在loading
     * 避免在业务代码中暴露类名
     * 1. 容器loading
     * 2. 按钮loading
     */

    $.fn.isLoading = function () {
        var container = $(this).eq(0);
        if (container.is('a') || container.is('label')) {
            return container.hasClass(CL_BUTTON_LOADING);
        }
        if (container.length == 0) {
            return false;
        }
        var dataLoading = container[0].loading;
        // loading图标元素
        var icon = dataLoading && dataLoading.el.icon;
        if (icon && icon.offset() && icon.offset().width > 0) {
            return true;
        }

        return false;
    };

    /**
     * 全屏是否正在loading快捷方法
     */
    $.isLoading = function () {
        return body.isLoading();
    };


	/**
     * 显示loading效果
	 * 配合CSS实现尺寸和透明度变化的动效
	 * 可用在任何Ajax异步操作呈现上，如：
	 * 1. 表格的分页请求；
	 * 2. ajax弹框呈现
	 * 支持jQuery 包装器调用和模块化调用
     */

    $.fn.loading = function (options) {
        return $(this).each(function () {
            var container = $(this);
            if (container.is('a') === false && container.is('label') === false) {
                container[0].loading = new Loading(container, options);
            } else {
                container.addClass(CL_BUTTON_LOADING);
            }
        });
    };


    /**
     * 全屏loading快捷方法
     */
    $.loading = function () {
        body.loading();
    };

	/**
     * unloading实际上是一个动画展示方法
	 * 会隐藏append进入的loading元素
    */
    $.fn.unloading = function (param) {
        var time = param || 0;
        if (typeof param != 'number') {
            time = 200;
        } else if (typeof param == 'undefined') {
            param = time;
        }

        return $(this).each(function (index, element) {
            var container = $(this);

            if (container.is('a') || container.is('label')) {
                container.removeClass(CL_BUTTON_LOADING);
                return;
            }

            // IE10+的游戏
            if (typeof history.pushState == 'function') {
                if (time > 0) {
					// 获得并存储当前的高度值
                    var height = container.height(), minHeight = container.css('min-height');
					// 以下行为瞬间执行，用户无感知
                    container.css({
                        height: 'auto',                // 高度设为auto, 用来获得此时的真实高度
                        webkitTransition: 'none',
                        transition: 'none',            // iOS safari下，'auto'也会触发transition过渡，因此，还原成'none'
                        overflow: 'hidden'             // 动画自然
                    });
					// 此时高度
                    var targetHeight = container.height();
					// 高度还原
                    container.height(height);
					// 移除动画
                    container.removeClass(CL_ANIMATION);
					// 触发重绘
                    element.offsetWidth;

					// 动画效果
					// 触发动画效果
                    if (param !== false) {
                        container.addClass(CL_ANIMATION);
                    }

					// 添加过渡效果
					// 过渡效果
                    container.css({
                        webkitTransition: 'height ' + time + 'ms',
                        transition: 'height ' + time + 'ms'
                    });

					// 终极尺寸
                    container.height(targetHeight);
                } else {
					// 过渡效果
                    container.css({
                        webkitTransition: 'none',
                        transition: 'none'
                    });

                    container.height('auto').removeClass(CL);
                }
            } else {
                container.height('auto');
            }
        });
    };

    /**
     * 全屏unloading快捷方法
     */
    $.unloading = function () {
        var loading = document.body.loading;
        if (loading) {
            loading.remove();
        }
    };

    var Loading = function (el, options) {
        var defaults = {
            text: false,          // 是否使用文本loading正在加载中...
            small: false,         // 是否是小菊花
            create: false         // loading是当前容器，还是append在当前容器中
        };

        var params = $.extend({}, defaults, options || {});

		// container是容器
		// loading是里面旋转的loading元素
		// 示意结构如下
		/* <div class='loading-x'>        <!-- 默认状态与结构 -->
 			  <i class='loading-icon'>

			<div class='loading-x'>
 			  <s class='loading-icon'>  <!-- 小尺寸图标 -->
		*/


        if (!el || (el === window || el === document) || (el.size && (el[0] === window || el[0] === document))) {
            el = body;
        }


		// 一般情况下，我们会直接在原始HTML就显示loading相关的HTML代码
		// 此时，我们其实什么都可以不用做
		// 我们只需要关心容器内没有loading的情况

		// 当然，也存在需要append创建的情况，如body中的loading等
        this._create = function () {
            var container = this.el.container;
            // loading元素，可能是container也可能是其子元素
            // icon一定是loading的相邻子元素
            var loading, icon;

            if (container.is('body')) {
                params.create = true;
            }

            // 1. 先判断是否需要生成对应的元素
            if (params.create == true) {
                loading = container.children('.' + CL);
                if (loading.size() == 0) {
                    container.append(loading = $('<div></div>').addClass(CL));
                }
            } else {
                loading = container;
            }

            // 是否有loading元素
            icon = loading.children('.' + CL_ICON);

            // 无则创建
            if (icon.size() == 0) {
				// 生成loading元素
                if (params.text == false) {
                    icon = (params.small ? $('<s></s>') : $('<i></i>')).addClass(CL_ICON).html(svg.replace('-svg"', '-svg-animate"') + svg);
                } else {
                    icon = $('<span></span>').addClass(CL_ICON).html('正在加载中<dot>...</dot>');
                }

				// 容器状态
                loading.empty().addClass(CL).append(icon);
				// ↑ 至此 loading效果已经出现
            }

            this.el.loading = loading;
            this.el.icon = icon;
        };

		// 元素存储
        this.el = {
            container: el,
            loading: null,
            icon: null
        };

        this.show();

        return this;
    };

    Loading.prototype.show = function () {
        var el = this.el;

        if (!el.loading || !el.icon) {
            this._create();
        }

        el.loading.show();

        this.display = true;

        return this;
    };

    Loading.prototype.hide = function () {
		// 需要判别loading和container是不是一个元素
		// 如果是，则隐藏图标
        var el = this.el, container = el.container, loading = el.loading;
        if (loading) {
            if (container.get(0) != loading.get(0)) {
                loading.hide();
            } else if (container.find('.' + CL_ICON).length) {
                loading.empty();
                this.el.icon = null;
            }
        }
        this.display = false;

        return this;
    };

    Loading.prototype.remove = function () {
		// remove方法移除元素以及类名，是比较彻底的清除
		// 如果是，则移除图标
        var el = this.el, container = el.container, loading = el.loading, icon = el.icon;

        if (loading && icon) {
            if (container.get(0) == loading.get(0)) {
                loading.removeClass(CL);
                icon.remove();
            } else {
                loading.remove();
            }

            this.el.loading = null;
            this.el.icon = null;
        }

        this.display = false;

        return this;
    };

    Loading.prototype.end = function (time) {
        var el = this.el, container = el.container;
        if (container) {
            container.unloading(time);
			// 标记当前的菊花态
            if (container.find('.' + CL_ICON).length == 0) {
                this.el.icon = null;
            }
        }

        this.display = false;

        return this;
    };

    return Loading;
}));
