/**
 * @Tab.js
 * @author zhangxinxu
 * @version
 * @Created: 15-06-12
 * @description form LuLu
 * @document  /guide
 */

(function (global, factory) {
    if (typeof exports === 'object' && typeof module !== 'undefined') {
        module.exports = factory();
    } else if (typeof define === 'function' && (define.amd || define.cmd)) {
        define(factory);
    } else {
        global.Tab = factory();
    }
}(this, function (require) {
    'use strict';

    /**
     * 定制级极简的选项卡
     * 选项卡效果本质上就是单选框切换效果
     * 因此，状态类名使用.checked
     * @使用示例
     *  new Tab($('#container > a'), {
     *     callback: function() {}
     *  });

     * 或者包装器用法
     * $('#container > a').tab({
     *   callback: function() {}
     * });
     */

    var STATE = 'active', ARIASELECTED = 'aria-selected';

    var Tab;

    // 根据属性获得对应的元素
    $.fn.tab = function (options) {
        new Tab($(this), options);
    };
    $.fn.eqAttr = function (key) {
        key = key || 'data-rel';
        // 获得属性对应的值
        var value = $(this).attr(key);
        if (!value) return $();
        // 当作id来处理
        var target = $('#' + value);
        if (target.length) return target;
        // 否则当作className处理
        return $('.' + value);
    };

    Tab = function (el, options) {
        var container = $(), line = $();
        // 选项卡元素，点击主体
        el = $(el);
        // 如果元素没有，打道回府
        if (el.length === 0) {
            return;
        }

        // 下面3语句参数获取
        options = options || {};
        var defaults = {
            eventType: 'click',
            history: 'auto',
            prepend: 'auto',
            index: 'auto',   // 'auto'表示根据选项卡上的状态类名，如.checked获取index值
            aria: 'a',    // 增加aria状态的元素向上查找的选择器
            callback: function () {}
        };
        var params = $.extend({}, defaults, options);

        // 元素处理
        if (el.length == 1 && el.is('a') == false) {
            container = el;
            el = container.find('a');

            if (params.history == 'auto') {
                params.history = true;
            }
            if (params.prepend == 'auto') {
                params.prepend = true;

                line = $('<line></line>');
                container.prepend(line);
            }
        }

        var self = this;

        self.callback = function (elTab) {
            if (params.history === true && history.pushState) {
                // 改变url地址，这样刷新的时候可以记住应该展开的选项卡是哪个
                // var key = 'tab', value = elTab.attr('data-rel');
                // if (value) {
                //     history.replaceState(null, document.title, url.setParam(key, value.toLowerCase()));
                // }
            }
            if (line.length) {
                line.css({
                    width: elTab.width(),
                    left: elTab.position().left
                });
            }

            params.callback.apply(this, arguments);
        };

        // 获取aria元素集
        var elAria = el.closest(params.aria).attr('role', 'tab');

        // 很重要的索引
        var indexTab = params.index;
        // 首先，获得索引值
        el.each(function (index) {
            if (typeof indexTab !== 'number' && $(this).hasClass(STATE)) {
                indexTab = index;
            }
            $(this).data('index', index);
        });

        if (typeof indexTab !== 'number') {
            indexTab = 0;
        }

        // 获得选项卡对应的面板元素
        // 两种情况
        // 1. 直接切换
        // 2. ajax并载入

        // 目前先第一种
        // 则直接就是页面元素
        // 事件gogogo
        el.on(params.eventType, function (event) {
            // 选项卡相关的一些元素
            var targetTab = $(this);
            var resetTab;
            // 面板元素
            var resetPanel, targetPanel;


            if (targetTab.hasClass(STATE)) {
                if (event.isTrusted === false || event.hasOwnProperty('_args')) {
                    // aria变化
                    elAria.attr(ARIASELECTED, 'false');
                    targetTab.closest(params.aria).attr(ARIASELECTED, 'true');
                    // 让可能不匹配的content面板的active状态同步
                    self.callback.call(this, targetTab, $(), $(), $());
                }
                return;
            }

            // 选项卡样式变变变
            // 要移除类名的选项卡
            resetTab = el.eq(indexTab).removeClass(STATE);
            // aria状态变化
            elAria.eq(indexTab).attr(ARIASELECTED, 'false');
            // 类名变化
            targetTab.addClass(STATE);
            // aria状态变化
            targetTab.closest(params.aria).attr(ARIASELECTED, 'true');

            // 面板的变化
            resetPanel = resetTab.eqAttr().removeClass(STATE);
            targetPanel = targetTab.eqAttr().addClass(STATE);


            // 索引参数变化
            indexTab = targetTab.data('index');

            // 回调方法
            self.callback.call(this, targetTab, resetTab, targetPanel, resetPanel);
        });


        // 默认就来一发
        $(function () {
            el.eq(indexTab).trigger(params.eventType);
        });

        // 暴露的属性或方法，供外部调用
        self.el = {
            tab: el
        };
        self.params = params;
    };

    return Tab;
}));
