/**
 * @Toggle.js
 * @author zhangxinxu
 * @version
 * @Created: 16-10-17
 * @description AB切换效果
 * @document  /guide
 */
define(function (require) {
    var $ = require('lib.Zepto');

    var ACTIVE = 'active', ENABLED = 'enabled';

    var Toggle = function (selector, options) {


        // 保存上下午
        var self = this;

        if (!selector) {
            return self;
        }

        if ($.isFunction(options)) {
            options = {
                callback: options
            };
        }

        var defaults = {
            mode: 'visible',    // 其他 more 展开收起
            container: $('body'),
            callback: function () {}
        };

        var params = $.extend({}, defaults, options || {});

        self.callback = params.callback;
        self.mode = params.mode;


        if (!selector.size && $.isArray(selector)) {
            selector = selector.join();
        }
        if (typeof selector == 'string') {
            params.container.on('click', selector, function (event) {
                self.toggle($(this), event);
            });
        } else if (selector.length) {
            selector.on('click', function (event) {
                self.toggle($(this), event);
            });
        }

        self.aria(selector);
    };

    Toggle.prototype.aria = function (selector) {
        var self = this;
        if (self.mode == 'visible' && typeof selector == 'string') {
            $(selector).each(function () {
                var trigger = $(this);
                trigger.attr({
                    role: 'menuitem',
                    'aria-expanded': trigger.hasClass(ACTIVE)
                });
            });
        }
    };

    Toggle.prototype.toggle = function (trigger, event) {
        var self = this;

        var target = trigger;
        var isActive;

        if (self.mode == 'visible') {
            target = $('#' + trigger.attr('data-rel'));

            // 当前是否展开
            isActive = trigger.hasClass(ACTIVE);

            if (isActive) {
                trigger.removeClass(ACTIVE).attr('aria-expanded', 'false');
                target.removeClass(ACTIVE);
            } else {
                trigger.addClass(ACTIVE).attr('aria-expanded', 'true');
                target.addClass(ACTIVE);
            }
        } else if (self.mode == 'more') {
            isActive = (typeof trigger.attr('open') == 'string');

            if (isActive) {
                trigger.removeAttr('open');
            } else {
                trigger.attr('open', '');
            }
        }

        self.callback.call(trigger, trigger, target, isActive, event);
    };

    return Toggle;
});
