/**
 * 滚动加载逻辑，提供滚动加载更多逻辑
 * 可以通过实现 onInfiniteScroll 来处理加载数据逻辑
 *
 * @author Zhang Chengrong
 *
 * @html
 * <div id="loadBar" class="module-bar"<%= data.isLast ? ' hidden' : '' %>>正在加载中...</div>
 */
define(function (require, exports, module) {
    var $ = require('lib.Zepto');

    // 滚动加载逻辑
    require('../../../../static/js/common/components/LazyLoad.js');

    var elTrigger = $('#loadBar');

    exports = module.exports = {
        scrollerSelector: null,

        // 过滤器初始化
        init: function () {
            this.initInfiniteScroll();
        },

        // 滚动加载更多数据
        initInfiniteScroll: function () {
            var self = this;
            var scrollerSelector = this.scrollerSelector || window;
            $(scrollerSelector).lazyLoad({
                selector: elTrigger.selector,
                callback: function () {
                    if (!elTrigger.attr('hidden') && !elTrigger.attr('disabled')) {
                        self.onInfiniteScroll();
                    }
                }
            });
        },

        // 触发滚动加载，需要被重写
        onInfiniteScroll: function () {
            console.warn('滚动加载'); // eslint-disable-line no-console
        },

        enableInfiniteScroll: function () {
            elTrigger.removeAttr('disabled');
        },

        disableInfiniteScroll: function () {
            elTrigger.attr('disabled', true);
        },

        hideInfiniteScroll: function () {
            elTrigger.attr('hidden', true);
        },

        showInfiniteScroll: function () {
            elTrigger.removeAttr('hidden');
        },

        toggleInfiniteScroll: function (isEnd) {
            if (isEnd) {
                this.hideInfiniteScroll();
            } else {
                this.showInfiniteScroll();
            }
        },
    };
});
