define(function (require, exports, module) {
    var $ = require('lib.Zepto');

    // 延迟触发包装器
    var debounce = require('../../../../static/js/common/utils/debounce.js');

    // SVG离线
    var requireSVG = require('../../../../static/js/common/utils/requireSVG.js');

    // 收起／展开控制器
    var Toggle = require('../../../../static/js/common/components/Toggle.js');

    // 最近阅读数据仓库
    var SearchHistory = require('../../../../static/js/common/store/searchHistory.js');

    // 全局提示
    require('../../../../static/js/common/components/Tips.js');

    // loading交互
    require('../../../../static/js/common/components/Loading.js');

    exports = module.exports = {
        /**
         * 页面初始化
         */
        init: function () {
            // 离线svg
            this.initSVG();

            this.reportEvent();

            this.backtop();

            this.setAjaxSettings();
        },

        setAjaxSettings: function () {
            $.ajaxSettings.timeout = 5 * 1000;
            $.ajaxSettings.error = function () {
                qdsdk.invokeClient('ui', 'showFailedPage');
            };
        },

        // 埋点点击上报事件绑定
        reportEvent: function () {
            $('body').on('click', '.report-ele', function (e) {
                var $ele = $(e.currentTarget);
                var eventId = $ele.data('eventid');
                var cbid = $ele.data('cbid');
                var groupid = $ele.data('groupid');
                var cli_url = $ele.data('cli_url');
                qdsdk.invokeClient('app', 'reportNative', {eventId: eventId, cbid: cbid, groupid: groupid, cli_url: cli_url});
            });

            var  reportpage = $('body').data('reportpage');
            if(reportpage) {
                qdsdk.invokeClient('app', 'reportNative', {eventId: reportpage});
            }
        },

        // 离线svg
        initSVG: function () {
            requireSVG('../../../static/icon/common/sprite.svg');
        },

        // 如果断网提醒用户
        checkNetwork: function () {
            if(!navigator.onLine)  {
                qdsdk.ui.toast('未连接网络，请检查您的网络连接');
            }
        },

        backtop: function () {
            var container = $(window);
            // 返回顶部按钮
            var elBacktop = $('.jsBackToTop').on('click', function () {
                var scrollTop = container.scrollTop();
                var steps = function () {
                    scrollTop *= 0.5;
                    if (scrollTop < 1) {
                        container.scrollTop(0);
                    } else {
                        container.scrollTop(scrollTop);
                        requestAnimationFrame(steps);
                    }
                };
                steps();
                return false;
            });

            container.on('scroll', function () {
                var st = container.scrollTop();
                var h = container.height();
                // 增加返回顶部和下载app模块区域交汇是隐藏的逻辑
                if (st > h) {
                    elBacktop.css({
                        opacity: 1,
                        visibility: 'visible'
                    });
                    return;
                }
                elBacktop.css({
                    opacity: 0,
                    visibility: 'hidden'
                });
            });
        }
    };
});
