/**
 * 利用 mixins 合并页面逻辑
 *
 * @author Zhang Chengrong
 */
define(function (require, exports, module) {
    // 策略集
    var strategys = {
        // 覆盖策略
        cover: function (parentVal, childVal) {
            return childVal === undefined ? parentVal : childVal;
        },

        // 合并策略
        merge: function (parentVal, childVal) {
            if (childVal) {
                if (parentVal) {
                    return parentVal.concat(childVal);
                }
                return Array.isArray(childVal) ? childVal : [childVal];
            }
            return parentVal;
        },
    };

    // 需要合并的方法
    var mergeItems = ['init'];

    // 页面基础对象
    var base = {
        _call: function (name) { // eslint-disable-line no-underscore-dangle
            var handlers = this['_' + name];

            if (handlers) {
                for (var i = 0, n = handlers.length; i < n; i++) {
                    handlers[i].call(this);
                }
            }
        },
    };

    function merge(parent, child) {
        var options = {};

        if (child.mixins) {
            child.mixins.forEach(function (mixin) {
                parent = merge(parent, mixin);
            });
        }

        function mergeField(key) {
            var strategy = mergeItems.indexOf(key) > -1 ? strategys.merge : strategys.cover;
            options[key] = strategy(parent[key], child[key]);
        }

        Object.keys(parent).forEach(mergeField);
        Object.keys(child).forEach(mergeField);

        return options;
    }

    exports = module.exports = function (options) {
        if (!options) {
            options = {};
        }

        var page = merge(base, options);

        mergeItems.forEach(function (key) {
            page['_' + key] = page[key];
            page[key] = function () {
                this._call(key); // // eslint-disable-line no-underscore-dangle
            };
        });

        return page;
    };
});
