define(function (require, exports, module) {
    /**
     * 延迟触发包装器
     * @param {Function} func 执行方法
     * @param {Number} wait 延迟时间
     * @return {Function} 包装之后的可调用方法
     */
    module.exports = function debounce(func, wait) {
        var context;
        var args;
        var result;
        var previous;
        var timer;

        var later = function () {
            var diff = Date.now() - previous;

            if (diff < wait && diff > 0) {
                timer = setTimeout(later, wait - diff);
            } else {
                timer = null;
                result = func.apply(context, args);

                if (!timer) {
                    context = args = null;
                }
            }
        };

        return function () {
            context = this;
            args = arguments;

            previous = Date.now();

            if (!timer) {
                timer = setTimeout(later, wait);
            }

            return result;
        };
    };
});
