define(function (require, exports, module) {
    /**
     * 间隔触发包装器
     * @param {Function} func 执行方法
     * @param {Number} wait 间隔时间
     * @return {Function} 包装之后的可调用方法
     */
    module.exports = function throttle(func, wait) {
        var context;
        var args;
        var result;
        var previous = 0;
        var timer = null;

        var later = function () {
            previous = Date.now();
            timer = null;

            result = func.apply(context, args);

            if (!timer) {
                context = args = null;
            }
        };

        return function () {
            context = this;
            args = arguments;

            var diff = Date.now() - previous;

            if (diff >= wait || diff < 0) {
                clearTimeout(timer);
                later();
            } else if (!timer) {
                timer = setTimeout(later, wait - diff);
            }

            return result;
        };
    };
});
