define(function(require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');
    // 延迟滚动加载
    var LazyLoad = require('../../../static/js/common/components/LazyLoad.js');

    // 滚动容器
    var container = $(g_data.container || window);

    exports = module.exports = {
        /**
         * 页面初始化
         */
        init: function() {
            // 倒计时
            // fill data
            this.fillData();
            this.refreshEvent();
        },

        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', {name: 'refreshEvent'});
            qdsdk.addEventListener('refreshEvent', function (result) {
                if(result.result == 0) {
                    // true表示是通过refresh触发的
                    self.fillData(true);
                }
            });
        },         
        // 限时免费倒计时效果
        initFree: function () {
            var $time = $('#time');

            $.getJSON(ajaxUrl + 'freeBook/getEndTime', {
            }, function (result) {
                // 元素
                // 子元素
                var elTimeChilds = $time.children();
                // 时间
                var seconds = result.endTime;
                // 定时器
                var timer = null;
                clearInterval(timer);

                // 加0方法，如果小于10
                var fnZero = function (number) {
                    if (number > 0) {
                        if (number <= 9) {
                            return '0' + number;
                        }
                        return '' + number;
                    }
                    return '00';
                };

                if (!seconds) {
                    seconds = 0;
                }

                // 显示时间
                var fnTime = function () {
                    if (seconds >= 0) {
                        var arrTime = [
                            fnZero(Math.floor(seconds / 86400)),
                            fnZero(Math.floor((seconds % 86400) / 3600)),
                            fnZero(Math.floor((seconds % 3600) / 60)),
                            fnZero(seconds % 60)
                        ];

                        elTimeChilds.each(function (index) {
                            if (arrTime[index] !== undefined) {
                                $(this).html(arrTime[index]);
                            }
                        });
                    } else {
                        elTimeChilds.html('00');
                        clearInterval(timer);
                    }

                    seconds -= 1;
                };

                fnTime();

                timer = setInterval(fnTime, 1000);

                // 显示时间
                $time.removeAttr('data-second');
            });
        },

        // 滚动加载
        initLazyLoad: function() {
            container.lazyLoad();
        }, 

        //  填充数据
        fillData: function(refreshEvent) {
            var self = this;
            $.getJSON(ajaxUrl + 'freeBook/getFreeBookInfo', function (result) {
                if (result.code == 0) {
                    var data = result.data;

                    if(refreshEvent) {
                        qdsdk.invokeClient('ui', 'cancelRefresh');
                        qdsdk.ui.toast('更新成功');
                    } else {
                        self.initFree();
                    }
                    
                    data.hotFree.eventid = 'hx_B41';
                    var hotFree = ejs.render('#tpl-book-list.html', { list: data.hotFree });
                    $('#hotFree').html(hotFree); // 人气免费

                    data.newFree.eventid = 'hx_B42';
                    var newFree = ejs.render('#tpl-book-list.html', { list: data.newFree });
                    $('#newFree').html(newFree); // 人气免费

                    // 滚动加载
                    self.initLazyLoad();
                }
            });

            $.getJSON(ajaxUrl + 'freeBook/getLimitFree', function (result) {
                if (result.code == 0) {
                    var bookList = result.data.bookList;
                    if (bookList != 0) {
                        var limitFreeBookInfo = ejs.render('#tpl-slide-li.html', { list: bookList });
                        $('#limitFreeBookInfo').html(limitFreeBookInfo); // 限时免费
                        $('#limit-module').show();
                    }
                }
            });
            qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
        }
    };
});
