define(function(require, exports, module) {
    var $ = require('lib.Zepto');
    // 延迟滚动加载
    var LazyLoad = require('../../../static/js/common/components/LazyLoad.js');
    var ejs = require('util.ejs2');

    // 滚动容器
    var container = $(g_data.container || window);

    exports = module.exports = {
        /**
         * 页面初始化
         */
        init: function() {
            // 倒计时
            this.initTimeDown();
            // fill data
            this.fillData();
            this.refreshEvent();
        },

        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', {name: 'refreshEvent'});
            qdsdk.addEventListener('refreshEvent', function (result) {
                if(result.result == 0) {
                    // true表示是通过refresh触发的
                    self.fillData(true);
                }
            });
        }, 

        // 倒计时
        initTimeDown: function() {
            // 元素
            var elTime = $('#time');
            // 子元素
            var elTimeChilds = elTime.children();
            // 时间
            var seconds = +elTime.attr('data-second');
            // 定时器
            var timer = null;

            // 加0方法，如果小于10
            var fnZero = function(number) {
                if (number > 0) {
                    if (number <= 9) {
                        return '0' + number;
                    }
                    return '' + number;
                }
                return '00';
            };

            if (!seconds) {
                seconds = 0;
            }

            // 显示时间
            var fnTime = function() {
                if (seconds >= 0) {
                    var arrTime = [
                        fnZero(Math.floor(seconds / 86400)),
                        fnZero(Math.floor((seconds % 86400) / 3600)),
                        fnZero(Math.floor((seconds % 3600) / 60)),
                        fnZero(seconds % 60)
                    ];

                    elTimeChilds.each(function(index) {
                        if (arrTime[index] !== undefined) {
                            $(this).html(arrTime[index]);
                        }
                    });
                } else {
                    elTimeChilds.html('00');
                    clearInterval(timer);
                }

                seconds -= 1;
            };

            fnTime();

            timer = setInterval(fnTime, 1000);

            // 显示时间
            elTime.removeAttr('data-second');

        },

        // 滚动加载
        initLazyLoad: function() {
            container.lazyLoad();
        }, 

        // Ajax拉取并填充数据
        fillData: function(refreshEvent) {
            var self = this;
            $.getJSON(ajaxUrl + 'book/getNewBookInfo', function (result) {
                if (result.code == 0) {
                    var data = result.data;
                    if(refreshEvent) {
                        qdsdk.invokeClient('ui', 'cancelRefresh');
                        qdsdk.ui.toast('更新成功');
                    }
                    data.dashenBook.eventid = 'hx_B43';
                    var dashenBook = ejs.render('#tpl-book-list.html', { list: data.dashenBook });
                    $('#dashenBook').html(dashenBook); // 大神新书

                    data.newBook.eventid = 'hx_B44';
                    var newBook = ejs.render('#tpl-book-list.html', { list: data.newBook });
                    $('#newBook').html(newBook); // 新书上架

                    data.hotBook.eventid = 'hx_B45';
                    var hotBook = ejs.render('#tpl-book-list.html', { list: data.hotBook });
                    $('#hotBook').html(hotBook); // 畅销新书

                    // 滚动加载
                    self.initLazyLoad();
                    qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                }
            });
        }
    };
});
