define(function(require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');
    var LazyLoad = require('../../../static/js/common/components/LazyLoad.js');
    // var jssdk2 = require('../../../static/js/common/utils/jssdk2.js');
    var infiniteScrollMixin = require('../../../static/js/common/mixin/infiniteScroll.js');
    var lazyLoadMixin = require('../../../static/js/common/mixin/lazyLoad.js');
    var merge = require('../../../static/js/common/page/merge.js');

    var container = $(g_data.container || window);

    exports = module.exports = merge({

        mixins: [infiniteScrollMixin, lazyLoadMixin],

        init: function () {
            this.fillData();
            this.refreshEvent();
            this.initTab();
            this.setHeader();
        },
        params: {
            rankId: 0,
            pageNum: 1,
            period: ''
        },

        setHeader: function () {
            qdsdk.invokeClient('ui', 'setHeaderRight', {text: '筛选'}, function () {
                var $selectBox = $('.select-box');
                if ($selectBox.hasClass('hide'))  {
                    $selectBox.removeClass('hide');
                } else {
                    $selectBox.addClass('hide');
                }
            });
        },

        initTab: function () {
            var self = this;
            $('body').delegate('a', 'click', function () {
                var elTarget = $(this);
                var ACTIVE = 'active';
                if (elTarget.parent('#rankNav').length > 0) {
                    var elNavs = elTarget.parent().find('a');
                    var elSections = $('.jsSection');
                    self.params.rankId = elTarget.attr('data-id');
                    self.params.pageNum = 1;
                    // todo 临时hack
                    if (self.params.rankId == 7 || self.params.rankId == 10 ) {
                        self.params.period = 3;
                    } else {
                        self.params.period = '';
                    }
                    /**/

                    var elDetailPage = $('#' + elTarget.attr('data-rel'));

                    elNavs.removeClass(ACTIVE);
                    elTarget.addClass(ACTIVE);
                    elSections.removeClass(ACTIVE);
                    elDetailPage.addClass(ACTIVE);

                    self.fillData();
                }
            });
        },

        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', {name: 'refreshEvent'});
            qdsdk.addEventListener('refreshEvent', function (result) {
                if(result.result == 0) {
                    self.fillData(true);
                }
            });
        },

        initLazyLoad: function() {
            container.lazyLoad();
        },

        fillData: function (refreshEvent, selectEvent) {
            var self = this;
            self.disableInfiniteScroll();
            $.getJSON(ajaxUrl + 'rank/list',self.params, function (result) {
                self.enableInfiniteScroll();
                if (result.code == 0) {
                    var data = result.data;
                    var list = ejs.render('#tpl-rank-list.html', { list: data.records });
                    if (data.pageNum == 1) {
                        $('#page' + self.params.rankId).html(list);
                    } else {
                        $('#page' + self.params.rankId).append(list);
                    }
                    if (refreshEvent) {
                        qdsdk.invokeClient('ui', 'cancelRefresh');
                        qdsdk.ui.toast('更新成功');
                    }
                    if (!refreshEvent && !selectEvent) {
                        var selectStr = ejs.render('#tpl-select-list.html', { list: data.period});
                        $('.select-ul').html(selectStr);
                        $('.select-ul li').on('click', function (e) {
                            e.preventDefault();
                            self.params.period = $(this).data('id');
                            self.params.pageNum= 1;
                            self.fillData(false, true);
                            $('.select-box').addClass('hide');
                            $('.aside-overlay').addClass('hide');
                            $('.select-ul li').removeClass('active');
                            $(this).addClass('active');
                        });
                        $('.aside-overlay').on('click', function (e) {
                            e.preventDefault();
                            $('.select-box').addClass('hide');
                            $('.aside-overlay').addClass('hide');
                        });
                    }
                    self.toggleInfiniteScroll(result.data.isLast);
                    self.initLazyLoad();
                    qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                }
            });
        },

        onInfiniteScroll: function () {
            var self = this;
            self.params.pageNum++;
            self.fillData();
        }

    });
});
