define(function (require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');
    var merge = require('readnovelm/js/common/page/merge.js');

    exports = module.exports = merge({
        /**
         * 页面初始化
         */
        init: function () {
            var self = this;
            self.visibilityEvent();
            self.fillData();
        },
        /**
         *  加载数据
         */
        fillData: function () {
            var self = this;
            qdsdk.invokeClient('app', 'checkLogin', {}, function (result) {
                if(result.data.loginState) {
                    $.getJSON(ajaxUrl + 'readDuration/index', function (result) {
                        if (result.code === 0) {
                            var preIndex = result.data.preIndex;
                            var totalMins = result.data.totalMins;
                            var totalAmount = result.data.totalAmount;
                            $('.total-num').html(totalMins);
                            var btnStatus = result.data.status;
                            if(btnStatus == 0) {
                                $('.btn-get').html('领取<span class="get-num"></span>红袖币')                                
                                $('.get-num').html(totalAmount);                                                
                            } else if(btnStatus == 1) {
                                $('.btn-get').html('已领取').addClass('disable');
                            } else if(btnStatus == 2) {
                                $('.btn-get').html('暂不足兑换').addClass('disable');                                
                            }
                            self.setCurves(self.getRate(totalMins));                    
                            for(var i = 0;i <= preIndex;i++) {
                                $('#time-dot' + i).removeClass('icon-time-dot-not').addClass('icon-time-dot-done');
                            }
                            self.setTree(totalMins);
                            self.getEvent();            
                        } 
                    });
                } else {
                    $('.total-num').html(0); 
                    $('.btn-get').html('登录领红袖币').on('click', function (e) {
                        e.preventDefault();
                        qdsdk.app.openLogin();  
                        self.setCurves(self.getRate(0));  
                        self.setTree(0);                        
                    });                
                }
            });
        },
        setCurves: function (rate) {
            var timeDoneSvg = $('#timeDoneSvg path')[0];
            var length = timeDoneSvg.getTotalLength();
            $(timeDoneSvg).attr('stroke-dasharray', rate * length + ',' + (length + 10));
        },
        setTree: function (time) {
            var treeIndex;
            if(time >= 0 && time < 360) {
                treeIndex = 1;
            } else if (time >= 360 && time < 720) {
                treeIndex = 2;                
            } else if (time >= 720 && time < 960) {
                treeIndex = 3;                
            } else {
                treeIndex = 4;                                
            }
            $('.time-tree').addClass('time-tree' + treeIndex);
        },
        getRate: function (time) {
            if (time <= 15) {
                return 0;
            } else if (time > 15 && time <= 25) {
                return 0.05;
            } else if (time > 25 && time < 45) {
                return 0.1;
            } else if (time == 45) {
                return 0.17;
            } else if (time > 45 && time <= 60) {
                return 0.22;
            } else if (time > 60 && time < 90) {
                return 0.27;
            } else if (time == 90) {
                return 0.33;
            } else if (time > 90 && time <= 120) {
                return 0.38;
            } else if (time > 120 && time < 180) {
                return 0.43;
            } else if (time == 180) {
                return 0.5;
            } else if (time > 180 && time <= 240) {
                return 0.55;
            } else if (time > 240 && time < 360) {
                return 0.6;
            } else if (time ==360) {
                return 0.67;
            } else if (time > 360 && time <= 480) {
                return 0.72;
            } else if (time > 480 && time < 720) {
                return 0.77;
            } else if (time == 720) {
                return 0.84;
            } else if (time > 720 && time < 800) {
                return 0.89;
            } else if (time > 800 && time < 960) {
                return 0.94;
            } else if (time >= 960) {
                return 1;
            } else {
                return 0;
            }
        },
        getEvent: function () {
            var self = this;
            $('.btn-get').on('click', function (e) {
                e.preventDefault();
                if($(this).hasClass('disable')) {
                    return;
                }
                $.ajax({
                    type: "post",
                    url: ajaxUrl + 'readDuration/receive',
                    dataType: "json",
                    success: function (response) {
                        if (response.code === 0) {
                            self.pop('领取成功', response.msg);  
                            self.fillData();              
                        } else {
                            qdsdk.ui.alert({ title: '领取失败', text: response.msg }, function (data) {
                            });              
                        }
                    }
                });
            });
        },
        visibilityEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', {name: 'browserVisibilityChange'});
            qdsdk.addEventListener('browserVisibilityChange', function (result) {
                if(result.data.visibility) {
                    self.fillData();
                }
            });
        },
        pop: function (title, content) {
            $('.pop-title').html(title);
            $('.pop-content').html(content);    
            $('.pop').show();
            $('.btn-ok').on('click', function (e) {
                e.preventDefault();
                $('.pop').hide();            
            });
        }
    });
});
