define(function (require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');
    var merge = require('../../../static/js/common/page/merge.js');

    require('readnovelm/js/common/utils/url.js');

    exports = module.exports = merge({
        init: function () {
            var self = this;
            self.fillData(false, true);
            self.refreshEvent();
        },
        data: {},
        refreshEvent: function () {
            var self = this;
            qdsdk.invokeClient('event', 'add', {name: 'refreshEvent'});
            qdsdk.addEventListener('refreshEvent', function (result) {
                if(result.result == 0) {
                    self.fillData(true);
                }
            });
        },
        fillData: function (refreshEvent, first) {
            var self = this;
            $.getJSON(ajaxUrl + 'checkin/getUserCheckInDetail', function (result) {
                if (result.code === 0) {
                    if (refreshEvent) {
                        qdsdk.invokeClient('ui', 'cancelRefresh');
                        qdsdk.ui.toast('更新成功');
                    }
                    // curPanel.html(ejs.render(tpl, result));
                    var data = self.data = result.data;
                    $('#date-span').html(data.date);
                    var checkinListStr = ejs.render('#tpl-check-list.html', { list: data.checkInLogList, restLotteryTime: self.data.restLotteryTime, missingCurrentTimes: self.data.missingCurrentTimes });
                    $('.signin-detail-box').html(checkinListStr);
                    $('#total-num').html(data.allTimes);
                    $('#week-num').html(data.currentTimes);
                    if (!data.missingCurrentTimes) {
                        $('.btn-signin-all').attr('disabled', 'disabled');
                    }
                    self.bindEvent(first);
                    qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                }
            });
        },
        bindEvent: function (first) {
            var self = this;
            if (first) {
                self.signinAllEvent();
            }
            self.resigninEvent();
            self.signinTodayEvent();
            self.getRewardEvent();
        },
        signinTodayEvent: function () {
            var self = this;
            $('.btn-signin-today').on('click', function (e) {
                e.preventDefault();
                $.getJSON(ajaxUrl + 'checkin/CheckIn', function (result) {
                    if (result.code === 0) {
                        // curPanel.html(ejs.render(tpl, result));
                        qdsdk.ui.toast(result.msg);
                        self.fillData();
                    } else {
                        qdsdk.ui.toast(result.msg);
                    }
                });
            });
        },
        resigninEvent: function () {
            var self = this;
            $('.btn-resignin').on('click', function (e) {
                e.preventDefault();
                var day = $(this).data('day');
                if (self.data.amount < self.data.reCheckInCost) {
                    // 余额不足，弹框提供充值入口
                    qdsdk.ui.popup({text: '需要花费' + self.data.reCheckInCost + '红袖币，您当前余额不足', buttons: ['取消', '去充值']}, function (result) {
                        if (result.data.action == 1) {
                            qdsdk.app.openRecharge();
                        }
                    });
                } else {
                    qdsdk.ui.confirm({text: '需要花费' + self.data.reCheckInCost + '红袖币,确定补签吗?'}, function (result) {
                        if (result.data.action == 1) {
                            $.getJSON(ajaxUrl + 'checkin/reCheckIn', { checkInDate: day}, function (result) {
                                if (result.code == 0) {
                                    // curPanel.html(ejs.render(tpl, result));
                                    qdsdk.ui.toast(result.msg);
                                    self.fillData();
                                } else if (result.code == 12) {
                                    // 余额不足，弹框提供充值入口
                                    qdsdk.ui.confirm({text: '余额不足，是否前去充值？'}, function (result) {
                                        if (result.data.action == 1) {
                                            qdsdk.app.openRecharge();
                                        }
                                    });
                                } else {
                                    qdsdk.ui.toast(result.msg);
                                }
                            });
                        }
                    });
                }
            });
        },
        signinAllEvent: function () {
            var self = this;
            console.log(111);
            $('.btn-signin-all').on('click', function (e) {
                e.preventDefault();
                console.log(222);
                if (self.data.missingCurrentTimes == 0) {
                    qdsdk.ui.toast('您本周期没有漏签哦，不需要补签');
                } else {
                    if (self.data.amount < self.data.reCheckInCost * self.data.missingCurrentTimes) {
                        // 余额不足，弹框提供充值入口
                        qdsdk.ui.popup({text: '需要花费' + self.data.reCheckInCost + '红袖币，您当前余额不足', buttons: ['取消', '去充值']}, function (result) {
                            if (result.data.action == 1) {
                                qdsdk.app.openRecharge();
                            }
                        });
                    } else {
                        qdsdk.ui.confirm({text: '需要花费' + self.data.reCheckInCost * self.data.missingCurrentTimes + '红袖币,确定补签吗'}, function (result) {
                            console.log(result.data.action);
                            if (result.data.action == 1) {
                                $.getJSON(ajaxUrl + 'checkin/reCheckInByBatch', function (result) {
                                    if (result.code == 0) {
                                        // curPanel.html(ejs.render(tpl, result));
                                        qdsdk.ui.toast(result.msg);
                                        self.fillData();
                                    } else if (result.code == 12) {
                                        // 余额不足，弹框提供充值入口
                                        qdsdk.ui.confirm({text: '余额不足，是否前去充值？'}, function (result) {
                                            if (result.data.action == 1) {
                                                qdsdk.app.openRecharge();
                                            }
                                        });
                                    } else {
                                        qdsdk.ui.toast(result.msg);
                                    }
                                });
                            }
                        });
                    }
                }
            });
        },
        getRewardEvent: function () {
            var self = this;
            $('.btn-get-award').on('click', function (e) {
                e.preventDefault();
                if (self.data.restLotteryTime > 0) {
                    qdsdk.invokeClient('app', 'openCheckInLotteryDialog', function () {
                        self.fillData();
                    });
                } else {
                    qdsdk.ui.toast('您的剩余抽奖次数为0，不能再抽奖啦');
                }
            });
        },
    });
});
