define(function (require, exports, module) {
    var $ = require('lib.Zepto');
    var ejs = require('util.ejs2');

    require('readnovelm/js/common/components/Tab.js');
    require('readnovelm/js/common/components/LazyLoad.js');
    require('readnovelm/js/common/utils/url.js');
    var merge = require('readnovelm/js/common/page/merge.js');
    var infiniteScrollMixin = require('readnovelm/js/common/mixin/infiniteScroll.js');
    var lazyLoadMixin = require('readnovelm/js/common/mixin/lazyLoad.js');

    exports = module.exports = merge({
        mixins: [lazyLoadMixin, infiniteScrollMixin],
        /**
         * 页面初始化
         */
        init: function () {
            var self = this;
            $('#recordNav').tab({
                callback: function (targetTab) {
                    var id = targetTab.attr('data-rel');
                    if (id === 'outRecordX') {
                        self.curRecord = 'outRecordX';
                        self.params.type = 2;
                    } else {
                        self.curRecord = 'inRecordX';
                        self.params.type = 1;
                    }
                    if (targetTab.data('loaded')) {
                        return;
                    }

                    if (id === 'outRecordX') {
                        self.params.pageNum = 1;
                        self.loadData();
                        targetTab.data('loaded', true);
                    }
                }
            });
            self.loadData();
        },
        counter: [1, 1],
        curRecord: 'inRecordX',
        curMonth: [g_data.chargeMonth, undefined],
        params: {
            pageNum: 1,
            type: 1
        },
        /**
          * 监听滚动
          */
        onInfiniteScroll: function () {
            this.params.pageNum = ++this.counter[this.params.type - 1];
            this.loadData();
        },
        /**
         *  加载数据
         */
        loadData: function () {
            var self = this;

            var curPanel = $('#' + this.curRecord + ' .panel');
            this.disableInfiniteScroll();

            $.getJSON(ajaxUrl + 'user/account', this.params, function (result) {
                self.enableInfiniteScroll();
                if (result.code === 0) {
                    var tpl = '#tempRecordIn';
                    var logs;
                    var balance = result.data.balance;
                    $('.record-coin-num strong').html(balance.allBalance);
                    $('.record-free-coin-num').html(balance.freeBalance);
                    if (balance.freeBalance > 0) {
                        $('.expired-date').html('，最早过期时间：<output class="">' + balance.expiredDate + '</output>');
                    }
                    if (self.curRecord === 'inRecordX') {
                        logs = result.data.chargeLogs;
                    } else {
                        logs = result.data.consumeLogs;
                        tpl = '#tempRecordOut';

                        $('#' + self.curRecord + '>span').hide();

                        if (self.params.pageNum === 1) {
                            if (logs.length === 0) {
                                var str = ejs.render(tpl, result);
                                curPanel.html(str);

                                return;
                            }
                            $('#' + self.curRecord + ' .jsInfiniteScroll').show();
                        }
                    }
                    var index = self.params.type - 1;
                    result.data.curMonth = self.curMonth[index];
                    self.curMonth[index] = logs.length ? logs[logs.length - 1].timeMonthYear : '';

                    //var str = ejs.render(tpl, result);
                    //curPanel.append(str);
                    var str = ejs.render(tpl, result);
                    curPanel.append(str);

                    self.toggleInfiniteScroll(result.data.page.isLast, $('#' + self.curRecord + '>.jsInfiniteScroll'));
                    if(result.data.page.isLast) {
                        $('#' + self.curRecord + '>.jsInfiniteScroll').hide();
                    }
                    self.updateLazyLoad();
                    qdsdk.invokeClient('ui', 'setLoadingProgressFinish');
                }
            });
        },
    });
});
