/**
 * @fileOverview
 * @author rainszhang
 * Created: 16-08-24
 */
(function(global, factory) {
    if (typeof define === 'function') {
        // 支持LBF加载
        if(typeof LBF === 'function'){
            LBF.define('qidian.report', function(){
                return factory(global);
            });
        }else{
            define(function() {
                return factory(global);
            });
        }
    } else {
        factory(global);
    }
}(this, function(window) {

    var doc = document;

    var $ = $ || jQuery || window.$ || window.jQuery;

    // local & dev & oa 上报到一个错误的地址，方便自测、测试，起点通过g_data传参，其他项目默认就是直接post到运营环境了
    var envType = (function(){
        try {
            return (g_data && g_data.envType || 'pro')  === 'pro' ?  '' : g_data.envType;
        } catch(e){
            return '';
        }
    })();

    // 请求路径
    var cgi = '//'+ envType + 'www.qidian.com/qreport?';

    // common
    var commons = {

        // 平台
        path: 'pclog',

        // 行为 A：点击 P：浏览
        logtype: 'A',

        // 当前页面url
        pageUrl: window.location.href,

        // 来源
        referer: document.referrer,

        // 分辨率横屏
        sw: screen.width,

        // 分辨率竖屏
        sh: screen.height,

        // 横坐标
        x: '',

        // 纵坐标
        y: ''
    };

    var Report = {
        /**
         * @method init
         * @param params object {}
         */
        init: function(params){

            var that = this;

            // 页面加载后单独发请求统计PV
            $(document).ready(function(e) {

                var url = cgi;

                // 起点中文网
                var defaults = {

                    // 行为 A：点击 P：浏览
                    logtype: 'P',

                    // 页面ID
                    pageid: (function(){
                        try {
                            return g_data.pageId;
                        } catch(e){
                            return '';
                        }
                    })(),

                    // 横坐标
                    x: '',

                    // 纵坐标
                    y: ''
                };

                var obj = $.extend({}, commons, defaults, params);

                // 合并url：http://www.qidian.com/qreport?path=pclog&logtype=P&pageid=qd_p_qidian&pageUrl=&referer=&x=&y=&sw=&sh=&
                $.each(obj, function (key, value) {
                    url = url + key + '=' + encodeURIComponent(value) + '&';
                });

                // 去除最后一个&
                url = url.substring(0, url.length - 1);

                createSender(url);

                that.send();

                // 老起点还有额外的上报逻辑，无论如何，先一起上报了
                obj.isQD && reportOldSiteData();
            });
        },

        /**
         * @method send
         * @param params object {}
         */
        send: function(params){

            //初始化时，防label与input联动造成的一次点击两次冒泡
            var initial = 0;

            $(document).off('click.Report');
            $(document).on('click.Report', function(e){

                var now = +new Date();

                // 如果当前点击与上一次点击的间隙小于100ms，说明是label与input联动造成的一次点击两次冒泡，因此只取上次点击的上报即可
                if (now - initial < 100) {
                    return;
                }

                //每次点击后将当前时间戳缓存
                initial = now;

                var target = $(e.target);

                var url = cgi;

                // 起点中文网
                var defaults = {

                    // 行为 A：点击 P：浏览
                    logtype: 'A',

                    // 页面ID
                    pageid: (function(){
                        try {
                            return g_data.pageId;
                        } catch(e){
                            return '';
                        }
                    })(),

                    // 页面模块标识
                    eventid: '',

                    // 书籍ID
                    bookid: '',

                    // 章节信息
                    chapterUrl: '',

                    // 标签名
                    tid: '',

                    // 列表序号
                    rankid: '',

                    // 广告素材id
                    qd_dd_p1: '',

                    // 横坐标
                    x: e.clientX + $('body').scrollLeft() || '',

                    // 纵坐标
                    y: e.clientY + $('body').scrollTop() || ''
                };

                var obj = $.extend({}, commons, defaults, params);

                var currentElement = target;

                while(currentElement.get(0) && currentElement.get(0).tagName != 'BODY'){

                    // 数据统计也采用冒泡层级来区分模块，会采用l1~l7来标识，l1代表最外层，html层级越往里，依次递增，l2, l3, l4……
                    for(var i=0; i<7; i++){
                        if(currentElement.data('l'+(i+1))){
                            obj['l'+(i+1)] = currentElement.data('l'+(i+1));
                            break;
                        }
                    }

                    // 如果获取到列表index，rankid在l7以内，最里层元素aid、bid、chapterurl、tid之外
                    if(currentElement.data('rankid')){
                        obj.rankid = currentElement.data('rankid');
                    }

                    /**
                     * ==================================================
                     * 以下是最里层元素，在同一层
                     * ==================================================
                     */

                    // 如果获取到模块ID
                    if(currentElement.data('aid')){
                        obj.eventid = currentElement.data('aid');
                    }

                    // 如果点击的是书籍
                    if(currentElement.data('bid')){
                        obj.bookid = currentElement.data('bid');
                    }

                    // 如果点击的是章节
                    if(currentElement.data('chapterurl')){
                        obj.chapterUrl = currentElement.data('chapterurl');
                    }

                    // 如果点击的是标签
                    if(currentElement.data('tid')){
                        obj.tid = currentElement.data('tid');
                    }

                    // 广告素材id，暂定是跳转url
                    if(currentElement.data('qd_dd_p1') && currentElement.data('qd_dd_p1') == 1){
                        obj.qd_dd_p1 = currentElement.get(0).href || '';
                    }

                    currentElement = currentElement.parent();
                }

                // 合并url：http://www.qidian.com/qreport?logtype=A&pageid=qd_p_qidian&pageUrl=&referer=&eventid=qd_A102&bookid=&chapterUrl=&tid=&rankid=&x=177&y=1142&sw=1440&sh=900&
                $.each( obj, function( key, value ) {
                    url = url + key + '=' + encodeURIComponent(value) + '&';
                });

                // 去除最后一个&
                url = url.substring(0, url.length-1);

                // 防刷
                obj.l1 = obj.l1 || '';
                if(obj.l1 == ''){
                    return;
                }

                createSender(url);

                // reset 默认参数值，不然如果上一次有自定义参数，参数会一直跟随
                params = null;
            })
        }
    };

    var Cookie = {
        /**
         * method get
         * @param name
         * @returns {null}
         */
        get: function(name){
            var carr = doc.cookie.match(new RegExp("(^| )" + name + "=([^;]*)(;|$)"));

            if (carr != null){
                return decodeURIComponent(carr[2]);
            }

            return null;
        }
    };

    /**
     * 创建老站的数据上报器
     * @method createOldSiteSender
     */
    function reportOldSiteData(){

        // 只有在cookie里面的stat_sessid不存在才会发这个请求
        if (Cookie.get('stat_sessid')) {
            return;
        }

        var url = '//uedas.qidian.com/statajax.aspx?';
        var obj = {

            // 操作行为
            opName: 'AddSessionUser',

            // cookie中的stat_gid
            globalId: Cookie.get('stat_gid') || '',

            // cookie中的cmfuToken
            curToken: Cookie.get('cmfuToken') || '',

            // 页面title
            pageTitle: document.title,

            // 来源referrer
            referer: document.referrer,

            // 页面url
            pageUrl: window.location.href,

            // 页面路径
            pagePathName: window.location.pathname,

            // 页面查询部分
            pageQueryString: window.location.search,

            // 页面域名
            host: window.location.host
        };

        // 合并url: //uedas.qidian.com/statajax.aspx?opName=AddSessionUser&globalId=&referrer=&pageTitle=&host=&pagePathName=&pageQueryString=&pageUrl=&topPageUrl=&isErrorPage=0&curToken=
        $.each( obj, function( key, value ) {
            url = url + key + '=' + encodeURIComponent(value) + '&';
        });

        // 去除最后一个&
        url = url.substring(0, url.length-1);

        createSender(url);
    }

    /**
     * 创建发送请求器
     * @method createSender
     * @param url 发送的请求
     */
    function createSender(url){
        var img = new Image();
        img.onload = img.onerror = function(){
            img = null;
        };
        img.src = url;
    }

    // 如果发现是直接引用模式，透给全局，污染了全局对象：Report
    if (typeof define !== 'function') {
        window.Report = Report;
    }

    return Report;
}));