/**
 * @fileOverview
 * @author renjiale
 * @version 1
 * Created: 16-1-7 15:20
 */

 /**
  * Components related to the development of qidian
  * @module qidian
  */
LBF.define('qidian.wxShare',function(){
    /**
     * Provide weixin share function in client side
     * @class wxShare
     * @namespace qidian
     * @constructor
     * @return {Function} setWeiXinShareConfig(wxservice, title, link, imgUrl, desc); reConfig(title, link, imgUrl, desc);
     */

    var PloyH5 = {
        /**
         * The function that loading jweixin-1.0.0.js.
         * @method loadJS
         * @param  {String}    url The url of jweixin-1.0.0.js.
         * @param  {Function}  success The callback function triggered after it successfully sent xhr to php side.
         */
        loadJS: function(url, success) {
            define = null;//防止jweixin-1.0.0.js与别的js冲突造成wx出现引用错误。
            require = null;//防止jweixin-1.0.0.js与别的js冲突造成wx出现引用错误。
            var js = document.createElement('script');
            js.src = url;
            js.type = "text/javascript";
            success = success || function() { };
            js.onload = js.onreadystatechange = function() {
                if (!this.readyState || 'loaded' === this.readyState || 'complete' === this.readyState) {
                    success();
                    this.onload = this.onreadystatechange = null;
                    // ɾ�����õĽű�
                    this.parentNode.removeChild(this);
                }
            }
            document.getElementsByTagName('head')[0].appendChild(js);
        },
        /**
         * The function that sending xhr to php side for getting appId, timeStamp, signature.
         * @method crossAjax
         */
        crossAjax: function() {
            var opt = arguments[0];
            if (opt) {
                if (typeof (FormData) == "undefined") {
                    if (typeof (opt.supportCallBack) == "undefined") {
                        return;
                    }
                    else {
                        opt.supportCallBack();
                        return;
                    }
                };
                opt.cache = opt.cache || false;
                opt.url = opt.url || '';
                opt.method = opt.method || "get";
                opt.timeout = opt.timeout || 10000;
                var data = opt.data;
                var xhr = new XMLHttpRequest();
                xhr.timeout = opt.timeout;

                var getCacheRandom = function() {
                    var date = new Date();
                    return date.getFullYear() + '' + date.getMonth() + '' + date.getDate() + '' + date.getHours() + '' + date.getMinutes() + '' + date.getSeconds() + '' + date.getMilliseconds();
                };

                if (opt.method.toLowerCase() == "get") {
                    if (data) {
                        opt.url += opt.url.indexOf("?") > -1 ? "&" : "?";
                        for (var item in data) {
                            opt.url += item + "=" + encodeURIComponent(data[item]) + "&";
                        }
                        if (opt.url.lastIndexOf("&") == opt.url.length - 1) {
                            opt.url = opt.url.substr(0, opt.url.length - 1);
                        }
                        if (!opt.cache) {
                            opt.url += "&_=" + getCacheRandom();
                        }
                    }
                    xhr.open("get", opt.url);
                    xhr.send();
                }
                else {
                    var formData = new FormData();
                    if (data) {
                        for (var item in data) {
                            formData.append(item, data[item]);
                        }
                    }
                    if (!opt.cache) {
                        formData.append("_", getCacheRandom());
                    }
                    xhr.open("post", opt.url);
                    xhr.send(formData);
                }

                xhr.onreadystatechange = function(e) {
                    if (xhr.readyState == 4) {
                        if (xhr.status == 200) {
                            if (typeof (opt.success) != "undefined") {
                                opt.success(JSON.parse(xhr.responseText))
                            }
                        }
                        else {
                            if (xhr.status != 0 && typeof (opt.error) != "undefined") {
                                opt.error(xhr)
                            }
                        }
                    }
                };
                xhr.ontimeout = function(e) {
                    if (typeof (opt.timeoutCallBack) != "undefined") {
                        opt.timeoutCallBack(e);
                    }
                };
            }
        },
        /**
         * The primary function configs API of weixin share triggered in weixin client side. It also summarizes the process of the interaction between client and server: Firstly load jweixin-1.0.0.js, then send xhr to php side and finally use the returned appId,signature,timestamp to config weixin share api.
         * @method setWeiXinShareConfig
         * @param  {String}    wxservice The entity want to use weixin share api, which must have appId and other data gotten from open.weixin.qq.com.
         * @param  {String}    title The title of the link you want to share.
         * @param  {String}    link The address of the link you want to share.
         * @param  {String}    imgUrl The url of the image shown besides the title when you touch share button.
         * @param  {String}      desc The description of the link you want to share.
         * @example
         *       LBF.use(['qidian.wxShare'],function(weixin){
        weixin.setWeiXinShareConfig("qidianzhongwenwang", //两个值，表示两个公众号（起点中文网和起点读书）。建议使用：qidianzhongwenwang
                "测试微信分享功能", //分享的标题
                "http://m.qidian.com/ploy/20150320h5/index.htm", //分享的地址
                "http://sta.qidian.com/zt/2015/26302/images/buy.png", //分享的图标
                "这是一个demo,用来测试微信分享功能！", function() {
                    alert("分享成功！");//分享成功的回调
                }, function() {
                    alert("分享取消！");//取消分享的回调
                });
          });
         */
        setWeiXinShareConfig: function(wxservice, title, link, imgUrl, desc) {
            if (navigator.userAgent.toLowerCase().match(/micromessenger/i) != "micromessenger") {//检查是不是在微信浏览器中
                return;
            }
            var succ = function() { }; var cancel = function() { };
            if (arguments != null && arguments.length >= 6) {
                succ = arguments[5];
            }
            if (arguments != null && arguments.length >= 7) {
                cancel = arguments[6];
            }
            PloyH5.loadJS("http://res.wx.qq.com/open/js/jweixin-1.0.0.js", function() {

                var data = {
                    ajaxMethod: "getwxqdzwwjsconfig"
                };
                var success = function(data) {
                    if (data && data.IsSuccess) {
                        wx.config({
                            debug: false,
                            appId: data.ReturnObject.appId,
                            timestamp: data.ReturnObject.timestamp,
                            nonceStr: data.ReturnObject.nonceStr,
                            signature: data.ReturnObject.signature,
                            jsApiList: [
                                'checkJsApi',
                                'onMenuShareTimeline',
                                'onMenuShareAppMessage',
                                'onMenuShareQQ',
                                'onMenuShareWeibo',
                                'chooseImage',
                                'previewImage',
                                'uploadImage',
                                'downloadImage',
                                'hideMenuItems',
                                'showMenuItems',
                                'hideAllNonBaseMenuItem',
                                'showAllNonBaseMenuItem',
                                'getNetworkType',
                                'openLocation',
                                'getLocation',
                                'hideOptionMenu',
                                'showOptionMenu',
                                'closeWindow',
                                'scanQRCode']
                        });
                        wx.ready(function() {
                            wx.onMenuShareTimeline({
                                title: title,
                                link: link,
                                imgUrl: imgUrl,
                                success: function() {
                                    succ();
                                },
                                cancel: function() {
                                    cancel();
                                }
                            });
                            wx.onMenuShareAppMessage({
                                title: title,
                                desc: desc,
                                link: link,
                                imgUrl: imgUrl,
                                type: '',
                                dataUrl: '',
                                success: function() {
                                    succ();
                                },
                                cancel: function() {
                                    cancel();
                                }
                            });
                            wx.onMenuShareQQ({
                                title: title,
                                desc: desc,
                                link: link,
                                imgUrl: imgUrl,
                                success: function() {
                                    succ();
                                },
                                cancel: function() {
                                    cancel();
                                }
                            });
                        });
                        wx.error(function(res) {

                        });
                    }
                };
                PloyH5.crossAjax({ data: data, success: success, url: 'http://m.qidian.com/ajax/index.ashx', timeout: 2000 });
            });
        },

        /**
         * The function that can reconfig the title,url address, image, and description of the link you want to share.
         * @method reConfig
         * @param  {String}   title The title of the link you want to share.
         * @param  {String}   link The link address you want to share.
         * @param  {String}   imgUrl The url of the image shown besides the title when you touch share button.
         * @param  {String}    desc  The description of the link you want to share.
         * @example
         *       LBF.use(['lib.jQuery','qidian.wxShare'],function($,weixin){
                $("#update").on("click",function(){
                     weixin.reConfig(//重新设置参数值
                    "11111测试微信分享功能11111！", //新的分享标题
                    "http://m.qidian.com/ploy/20150320h5/index.htm", //新的分享地址
                    "http://mat1.gtimg.com/book/2015images/26340/code1.jpg", //新的分享图标
                    "2222这是一个demo,用来测试微信分享功能2222！",  //新的分享描述
                    function() {
                        alert("分享成功+1！");//新的成功回调
                    }, function() {
                        alert("分享取消+1！");//新的取消分享回调
                    });
                alert("设置完成！");
            });
         });
         */
        reConfig: function(title, link, imgUrl, desc) {
            try {
                if (typeof wx != "undefined") {
                    var succ = function() { }; var cancel = function() { };
                    if (arguments != null && arguments.length >= 5) {
                        succ = arguments[4];
                    }
                    if (arguments != null && arguments.length >= 6) {
                        cancel = arguments[5];
                    }

                    wx.onMenuShareTimeline({
                        title: title,
                        link: link,
                        imgUrl: imgUrl,
                        success: function() {
                            succ();
                        },
                        cancel: function() {
                            cancel();
                        }
                    });
                    wx.onMenuShareAppMessage({
                        title: title,
                        desc: desc,
                        link: link,
                        imgUrl: imgUrl,
                        type: '',
                        dataUrl: '',
                        success: function() {
                            succ();
                        },
                        cancel: function() {
                            cancel();
                        }
                    });
                    wx.onMenuShareQQ({
                        title: title,
                        desc: desc,
                        link: link,
                        imgUrl: imgUrl,
                        success: function() {
                            succ();
                        },
                        cancel: function() {
                            cancel();
                        }
                    });
                }
            }
            catch (e) {

            }
        }

    };
    return {
        setWeiXinShareConfig:PloyH5.setWeiXinShareConfig,
        reConfig:PloyH5.reConfig
    }
});

